__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, equal_to

from tests.base_tests import WxWidgetTestCase
from ossos.gui.views.listctrls import ListCtrlPanel


class ListViewTest(WxWidgetTestCase):
    def setUp(self):
        super(ListViewTest, self).setUp()

        self.dataset1 = (("Key1", "Val1"), ("Key2", "Val2"), ("Key3", "Val3"))

        self.view = ListCtrlPanel(self.rootframe, ("Col1", "Col2"))

    def test_kvlist_displayed(self):
        self.view.populate_list(self.dataset1)

        assert_that(self.view.list.GetColumnCount(), equal_to(2))
        assert_that(self.view.list.GetItemCount(), equal_to(3))

        # TODO create custom matcher?
        for item_ind in range(self.view.list.GetItemCount()):
            for col_ind in range(self.view.list.GetColumnCount()):
                assert_that(self.view.list.GetItem(item_ind, col_ind).GetText(),
                            equal_to(self.dataset1[item_ind][col_ind]))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, equal_to

from ossos import coding


class EncodingDecodingTest(unittest.TestCase):
    def test_base36_encode_decode_1(self):
        assert_that(coding.base36encode(1), equal_to("1"))
        assert_that(coding.base36decode("1"), equal_to(1))

    def test_base36_encode_10(self):
        assert_that(coding.base36encode(10), equal_to("A"))
        assert_that(coding.base36decode("A"), equal_to(10))

    def test_base36_encode_100(self):
        assert_that(coding.base36encode(100), equal_to("2S"))
        assert_that(coding.base36decode("2S"), equal_to(100))

    def test_base36_encode_10000(self):
        assert_that(coding.base36encode(10000), equal_to("7PS"))
        assert_that(coding.base36decode("7PS"), equal_to(10000))

    def test_base36_encode_pad_short(self):
        assert_that(coding.base36encode(1, pad_length=2), equal_to("01"))

    def test_base36_encode_pad_long(self):
        assert_that(coding.base36encode(10000, pad_length=2), equal_to("7PS"))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from astropy.io.fits.hdu.hdulist import HDUList
import matplotlib.pyplot as plt
from hamcrest import assert_that, close_to, equal_to, none
from mock import Mock, MagicMock

from ossos import astrom
from ossos.downloads.cutouts.source import SourceCutout
from ossos.downloads.cutouts.grid import CutoutGrid
from ossos.fitsviewer.colormap import clip
from ossos.fitsviewer.displayable import (DisplayableImageSinglet,
                                          DisplayableImageTriplet,
                                          ImageSinglet)
from ossos.fitsviewer import displayable


class ImageSingletTest(unittest.TestCase):
    def setUp(self):
        mainhdu = Mock()
        mainhdu.data.shape = (100, 100)
        self.hdulist = [mainhdu]
        fig = plt.figure()

        self.displayable = ImageSinglet(self.hdulist, fig, [0, 0, 1, 1])

    def test_draw_one_circle(self):
        assert_that(self.displayable.marker, none())

        cx = 1
        cy = 2
        cr = 3
        self.displayable.place_marker(cx, cy, cr)

        assert_that(self.displayable.marker.center, equal_to((cx, cy)))
        assert_that(self.displayable.marker.radius, equal_to(cr))

    def test_draw_second_circle_removes_first(self):
        c1x = 1
        c1y = 2
        c1r = 3
        self.displayable.place_marker(c1x, c1y, c1r)

        assert_that(self.displayable.marker.center, equal_to((c1x, c1y)))
        assert_that(self.displayable.marker.radius, equal_to(c1r))

        c2x = 4
        c2y = 5
        c2r = 6
        self.displayable.place_marker(c2x, c2y, c2r)

        assert_that(self.displayable.marker.center, equal_to((c2x, c2y)))
        assert_that(self.displayable.marker.radius, equal_to(c2r))

    def test_toggle_reticule_notifies_display_changed(self):
        self.displayable.place_marker(10, 10, 10)

        refresh_handler = Mock()
        self.displayable.display_changed.connect(refresh_handler)

        self.displayable.toggle_reticule()

        refresh_handler.assert_called_once_with()


class DisplayableImageSingletTest(unittest.TestCase):
    def setUp(self):
        self.singlet = DisplayableImageSinglet(MagicMock())

    def mock_image_singlet(self):
        image_singlet = Mock(spec=ImageSinglet)
        self.singlet.image_singlet = image_singlet
        return image_singlet

    def test_reset_colormap(self):
        image_singlet = self.mock_image_singlet()

        self.singlet.reset_colormap()
        image_singlet.reset_colormap.assert_called_once_with()

    def test_toggle_reticule(self):
        image_singlet = self.mock_image_singlet()

        self.singlet.toggle_reticule()
        image_singlet.toggle_reticule.assert_called_once_with()


class DisplayableImageTripletTest(unittest.TestCase):
    def setUp(self):
        source = Mock(spec=astrom.Source)
        source.num_readings.return_value = 3

        def mock_hdulist():
            return MagicMock(spec=HDUList)

        grid = CutoutGrid(source)
        self.hdulist00 = mock_hdulist()
        self.hdulist01 = mock_hdulist()
        self.hdulist02 = mock_hdulist()
        self.hdulist10 = mock_hdulist()
        self.hdulist11 = mock_hdulist()
        self.hdulist12 = mock_hdulist()
        self.hdulist20 = mock_hdulist()
        self.hdulist21 = mock_hdulist()
        self.hdulist22 = mock_hdulist()

        def mock_cutout(hdulist):
            cutout = Mock(spec=SourceCutout)
            cutout.hdulist = hdulist
            return cutout

        grid.add_cutout(mock_cutout(self.hdulist00), 0, 0)
        grid.add_cutout(mock_cutout(self.hdulist01), 0, 1)
        grid.add_cutout(mock_cutout(self.hdulist02), 0, 2)
        grid.add_cutout(mock_cutout(self.hdulist10), 1, 0)
        grid.add_cutout(mock_cutout(self.hdulist11), 1, 1)
        grid.add_cutout(mock_cutout(self.hdulist12), 1, 2)
        grid.add_cutout(mock_cutout(self.hdulist20), 2, 0)
        grid.add_cutout(mock_cutout(self.hdulist21), 2, 1)
        grid.add_cutout(mock_cutout(self.hdulist22), 2, 2)

        self.grid = grid

    def test_frames_have_correct_hdulists(self):
        displayable = DisplayableImageTriplet(self.grid)

        def get_hdulist(frame_index, time_index):
            return displayable.get_singlet(frame_index, time_index).hdulist

        assert_that(get_hdulist(0, 0), equal_to(self.hdulist00))
        assert_that(get_hdulist(0, 1), equal_to(self.hdulist01))
        assert_that(get_hdulist(0, 2), equal_to(self.hdulist02))

        assert_that(get_hdulist(1, 0), equal_to(self.hdulist10))
        assert_that(get_hdulist(1, 1), equal_to(self.hdulist11))
        assert_that(get_hdulist(1, 2), equal_to(self.hdulist12))

        assert_that(get_hdulist(2, 0), equal_to(self.hdulist20))
        assert_that(get_hdulist(2, 1), equal_to(self.hdulist21))
        assert_that(get_hdulist(2, 2), equal_to(self.hdulist22))


class UtilityTest(unittest.TestCase):
    def assert_close(self, expected, actual):
        assert_that(expected, close_to(actual, 0.0001))

    def test_clip_in_range(self):
        assert_that(clip(0.5, 0, 1), equal_to(0.5))

    def test_clip_below_range(self):
        assert_that(clip(-0.5, 0, 1), equal_to(0.0))

    def test_clip_above_range(self):
        assert_that(clip(1.5, 0, 1), equal_to(1.0))

    def test_get_rect_first_frame_first_time_top_left(self):
        [left, bottom, width, height] = displayable.get_rect((3, 3), 0, 0,
                                                             border=0, spacing=0)
        assert_that(left, equal_to(0))
        assert_that(bottom, equal_to(2./3))
        assert_that(width, equal_to(1./3))
        assert_that(height, equal_to(1./3))

    def test_get_rect_last_frame_last_time_bottom_right(self):
        [left, bottom, width, height] = displayable.get_rect((3, 3), 2, 2,
                                                             border=0, spacing=0)
        assert_that(left, equal_to(2./3))
        assert_that(bottom, equal_to(0))
        assert_that(width, equal_to(1./3))
        assert_that(height, equal_to(1./3))

    def test_get_rect_last_frame_first_time_bottom_left(self):
        [left, bottom, width, height] = displayable.get_rect((3, 3), 2, 0,
                                                             border=0, spacing=0)
        assert_that(left, equal_to(0))
        assert_that(bottom, equal_to(0))
        assert_that(width, equal_to(1./3))
        assert_that(height, equal_to(1./3))

    def test_get_rect_last_frame_first_time_with_border(self):
        [left, bottom, width, height] = displayable.get_rect((3, 3), 2, 0,
                                                             border=0.05, spacing=0)
        assert_that(left, equal_to(0.05))
        assert_that(bottom, equal_to(0.05))
        assert_that(width, equal_to(0.3))
        assert_that(height, equal_to(0.3))

    def test_get_rect_mid_frame_first_time_with_spacing(self):
        [left, bottom, width, height] = displayable.get_rect((3, 3), 1, 0,
                                                             border=0, spacing=0.05)
        assert_that(left, equal_to(0))
        assert_that(bottom, equal_to(0.35))
        assert_that(width, equal_to(0.3))
        assert_that(height, equal_to(0.3))

    def test_get_rect_mid_frame_first_time_with_spacing_and_border(self):
        [left, bottom, width, height] = displayable.get_rect((3, 3), 1, 0,
                                                             border=0.025, spacing=0.025)

        self.assert_close(left, 0.025)
        self.assert_close(bottom, 0.35)
        self.assert_close(width, 0.3)
        self.assert_close(height, 0.3)


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from mock import Mock, ANY, patch
from hamcrest import assert_that, equal_to, is_not, same_instance

from tests.base_tests import FileReadingTestCase, WxWidgetTestCase, DirectoryCleaningTestCase
from ossos.astrom import AstromParser
from ossos.daophot import TaskError
from ossos.downloads.cutouts.source import SourceCutout
from ossos.gui import tasks
from ossos.gui.context import LocalDirectoryWorkingContext
from ossos.gui.progress import LocalProgressManager
from ossos.gui.controllers import ProcessRealsController
from ossos.gui.models.imagemanager import ImageManager
from ossos.gui.models.validation import ValidationModel
from ossos.gui.views.appview import ApplicationView
from ossos import mpc
from ossos.naming import ProvisionalNameGenerator
from ossos.gui.models.workload import WorkUnitProvider, RealsWorkUnitBuilder

TEST_MINOR_PLANET_NUMBER = "mpn01"
TEST_PROVISIONAL_NAME = "DNZOH00"
TEST_DISCOVERY_AST = "*"
TEST_NOTE1 = "A  earlier approximate position inferior"
TEST_NOTE2 = "C   CCD"
TEST_DATE = "2013 04 09.43325"
TEST_DEC = 31.2123
TEST_RA = 27.213
TEST_MAG = "123.5"
TEST_MAG_ERR = "5"
TEST_BAND = "A"
TEST_OBS_CODE = "523"
TEST_COMMENT = "Test comment"


class ProcessRealsControllerTest(WxWidgetTestCase, FileReadingTestCase, DirectoryCleaningTestCase):
    def setUp(self):
        WxWidgetTestCase.setUp(self)

        parser = AstromParser()
        context = LocalDirectoryWorkingContext(
            self.get_abs_path("data/controller_testdir"))
        progress_manager = LocalProgressManager(context)
        workunit_provider = WorkUnitProvider(tasks.get_suffix(tasks.REALS_TASK),
                                             context, progress_manager,
                                             RealsWorkUnitBuilder(
                                                 parser,
                                                 context,
                                                 context,
                                                 progress_manager))

        image_manager = Mock(spec=ImageManager)

        self.model = ValidationModel(workunit_provider, image_manager, None)
        self.model.start_work()

        self.model.get_writer = Mock(return_value=Mock(spec=mpc.MPCWriter))

        # We don't actually have any images loaded, so mock this out
        source_cutout = Mock(spec=SourceCutout)
        source_cutout.pixel_x = 11
        source_cutout.pixel_y = 50
        self.model.get_current_cutout = Mock(return_value=source_cutout)

        x_cen = 10
        y_cen = 50
        mag = 24.0
        self.model.get_current_source_observed_magnitude = Mock(return_value=(x_cen, y_cen, mag))

        self.model.is_current_source_adjusted = Mock(return_value=False)
        self.model.get_current_fits_header = Mock()

        self.name_generator = Mock(spec=ProvisionalNameGenerator)
        self.name_generator.generate_name.return_value = TEST_PROVISIONAL_NAME

        class TestFactory(object):
            def __init__(self, model, name_generator):
                self.model = model
                self.name_generator = name_generator

            def create_controller(self, view):
                return ProcessRealsController(self.model, view,
                                              self.name_generator)

        self.view = ApplicationView(
            TestFactory(self.model, self.name_generator))
        self.controller = self.view.controller
        self.controller.display_current_image = Mock()

    def tearDown(self):
        WxWidgetTestCase.tearDown(self)
        DirectoryCleaningTestCase.tearDown(self)

    def get_directory_to_clean(self):
        return self.get_abs_path("data/controller_testdir")

    def get_files_to_keep(self):
        return ["1584431p15.measure3.reals.astrom", "1616681p10.measure3.reals.astrom"]

    @patch("ossos.gui.controllers.mpc.Observation", spec=mpc.Observation)
    def test_reject_disables_validation_controls(self, mock_Observation):
        comment = "test"

        assert_that(self.view.is_source_validation_enabled(), equal_to(True))
        self.controller.on_do_reject(comment)

        # We have moved to the next item, so it should still be enabled
        assert_that(self.view.is_source_validation_enabled(), equal_to(True))

        # Loop back to that first item
        self.controller.on_next_obs()
        self.controller.on_next_obs()
        assert_that(self.view.is_source_validation_enabled(), equal_to(False))

        self.controller.on_next_obs()
        assert_that(self.view.is_source_validation_enabled(), equal_to(True))
        self.controller.on_next_obs()
        assert_that(self.view.is_source_validation_enabled(), equal_to(True))

    @patch("ossos.gui.controllers.mpc.Observation", spec=mpc.Observation)
    def test_reject_last_item_disables_validation_controls(self, mock_Observation):
        comment = "test"

        self.controller.on_next_obs()
        self.controller.on_next_obs()
        assert_that(self.view.is_source_validation_enabled(), equal_to(True))
        self.controller.on_do_reject(comment)

        # We have moved to the next item (looped back to beginning), so it should still be enabled
        assert_that(self.view.is_source_validation_enabled(), equal_to(True))

        # Move forward to that last item again
        self.controller.on_next_obs()
        self.controller.on_next_obs()
        assert_that(self.view.is_source_validation_enabled(), equal_to(False))

        self.controller.on_next_obs()
        assert_that(self.view.is_source_validation_enabled(), equal_to(True))
        self.controller.on_next_obs()
        assert_that(self.view.is_source_validation_enabled(), equal_to(True))

    def accept_source_reading(self):
        self.controller.on_do_accept(TEST_MINOR_PLANET_NUMBER,
                                     TEST_PROVISIONAL_NAME,
                                     TEST_NOTE1,
                                     TEST_NOTE2,
                                     TEST_DATE,
                                     TEST_RA,
                                     TEST_DEC,
                                     TEST_MAG,
                                     TEST_MAG_ERR,
                                     TEST_BAND,
                                     TEST_OBS_CODE,
                                     TEST_COMMENT
        )

    def reject_source_reading(self):
        self.controller.on_do_reject(TEST_COMMENT)

    def test_accept_moves_to_first_observation_of_new_workunit(self):
        workunit1 = self.model.get_current_workunit()

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.accept_source_reading()

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(1))

        self.accept_source_reading()

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(2))

        self.accept_source_reading()

        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.accept_source_reading()

        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_obs_number(), equal_to(1))

        self.accept_source_reading()

        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_obs_number(), equal_to(2))

        self.accept_source_reading()

        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.accept_source_reading()

        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_obs_number(), equal_to(1))

        self.accept_source_reading()

        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_obs_number(), equal_to(2))

        self.accept_source_reading()

        workunit2 = self.model.get_current_workunit()

        assert_that(workunit2, is_not(same_instance(workunit1)))

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

    def test_phot_error_handling(self):
        view_mock = Mock(spec=ApplicationView)
        self.controller.view = view_mock

        error_message = "Photometry failed"
        self.model.get_current_source_observed_magnitude = Mock(
            side_effect=TaskError(error_message))
        self.model.get_current_band = Mock(return_value="B")

        self.controller.on_accept()

        view_mock.show_accept_source_dialog.assert_called_once_with(
            ANY,
            ANY,
            ANY,
            ANY,
            "", # obs_mag
            -1, # obs_mag_err
            "", # band
            note1_choices=ANY,
            note2_choices=ANY,
            note1_default=ANY,
            note2_default=ANY,
            default_observatory_code=ANY,
            default_comment=error_message,
            phot_failure=True
        )


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Slider

from ossos.fitsviewer.colormap import GrayscaleColorMap


def create_test_image():
    x = np.linspace(0, 2 * np.pi, 120)
    y = np.linspace(0, 2 * np.pi, 100).reshape(-1, 1)
    return np.sin(x) + np.cos(y)


def main():
    img = create_test_image()
    colormap = GrayscaleColorMap()

    plt.subplot(1, 1, 1)
    plt.subplots_adjust(left=0.15, bottom=0.25)

    axcolor = 'lightgoldenrodyellow'
    contrast_ax = plt.axes([0.15, 0.15, 0.65, 0.03], axisbg=axcolor)
    bias_ax = plt.axes([0.15, 0.1, 0.65, 0.03], axisbg=axcolor)

    contrast_slider = Slider(contrast_ax, "Contrast", 0.0, 1.0, valinit=0.5)
    bias_slider = Slider(bias_ax, "Bias", 0.0, 1.0, valinit=0.5)

    cmap = colormap.as_mpl_cmap()
    plt.subplot(1, 1, 1)
    img_ax = plt.imshow(img, cmap=cmap)
    plt.colorbar(orientation="horizontal")

    def update_colormap():
        cmap = colormap.as_mpl_cmap()
        img_ax.set_cmap(cmap)

    def update_contrast(contrast):
        print "Setting contrast to %f" % contrast
        colormap.set_contrast(contrast)
        update_colormap()

    def update_bias(bias):
        print "Setting bias to %f" % bias
        colormap.set_bias(bias)
        update_colormap()

    contrast_slider.on_changed(update_contrast)
    bias_slider.on_changed(update_bias)

    plt.show()


if __name__ == "__main__":
    main()



__author__ = 'Michele Bannister   git:@mtbannister'

import numpy as np
import matplotlib
import matplotlib.pyplot as plt

from mpld3 import plugins, utils
import brewer2mpl

ALMOST_BLACK = '#262626'

# nicer defaults
# every colourbrewer scale: http://bl.ocks.org/mbostock/5577023
# hues create primary visual differences between classes; does not imply magnitude differences between classes
set2 = brewer2mpl.get_map('Set2', 'qualitative', 8).mpl_colors
# sequential for ordered data where light is low values, dark is high values
ylorrd = brewer2mpl.get_map('YlOrRd', 'sequential', 9).mpl_colors
# diverging for equal emphasis on mid-range critical values and extremes at both ends of data range
puor = brewer2mpl.get_map('PuOr', 'diverging', 11).mpl_colors

# FIXME: figure out how to export these to a figure

# rcParams['figure.figsize'] = (10, 10)
# rcParams['figure.dpi'] = 150
# rcParams['axes.color_cycle'] = puor
matplotlib.rcParams['font.size'] = 12  # good for posters/slides
matplotlib.rcParams['patch.facecolor'] = set2[0]
matplotlib.rcParams['font.family'] = 'sans-serif'
matplotlib.rcParams['font.sans-serif'] = ['Sofia Pro', 'Tahoma']


def remove_border(axes=None, keep=('left', 'bottom'), remove=('right', 'top'), labelcol=ALMOST_BLACK):
    """
    Minimize chart junk by stripping out unnecessary plot borders and axis ticks.
    The top/right/left/bottom keywords toggle whether the corresponding plot border is drawn
    """
    ax = axes or plt.gca()
    for spine in remove:
        ax.spines[spine].set_visible(False)
    for spine in keep:
        ax.spines[spine].set_linewidth(0.5)
        # ax.spines[spine].set_color('white')

    # remove all ticks, then add back the ones in keep
    # Does this also need to specify the ticks' colour, given the axes/labels are changed?
    ax.yaxis.set_ticks_position('none')
    ax.xaxis.set_ticks_position('none')
    # ax.xaxis.set_ticklabels("")
    # ax.yaxis.set_ticklabels("")

    for spine in keep:
        if spine == 'top':
            ax.xaxis.tick_top()
        if spine == 'bottom':
            ax.xaxis.tick_bottom()
            # match the label colour to that of the axes
            ax.xaxis.label.set_color(labelcol)
            ax.xaxis.set_tick_params(color=labelcol, labelcolor=labelcol)
        if spine == 'left':
            ax.yaxis.tick_left()
            ax.yaxis.label.set_color(labelcol)
            ax.yaxis.set_tick_params(color=labelcol, labelcolor=labelcol)
        if spine == 'right':
            ax.yaxis.tick_right()

    return


def clean_legend():
    # as per prettyplotlib.
    # Remove the line around the legend box, and instead fill it with a light grey
    # Also only use one point for the scatterplot legend because the user will
    # get the idea after just one, they don't need three.
    light_grey = np.array([float(248) / float(255)] * 3)
    legend = ax.legend(frameon=True, scatterpoints=1)
    rect = legend.get_frame()
    rect.set_facecolor(light_grey)
    rect.set_linewidth(0.0)

    # Change the legend label colors to almost black, too
    texts = legend.texts
    for t in texts:
        t.set_color(almost_black)


class LinkedBrush(plugins.PluginBase):
    JAVASCRIPT = r"""
    var LinkedBrushPlugin = function(fig, prop){
      this.fig = fig;
      this.prop = mpld3.process_props(this, prop, {}, ["id"]);

      var brush_plug = this;

      mpld3.Toolbar.prototype.buttonDict["brush"] = mpld3.ButtonFactory({
        onClick: this.onClick.bind(this),
        draw: function(){
            mpld3.BaseButton.prototype.draw.apply(this);
            var enable_zoom = brush_plug.fig.enable_zoom.bind(brush_plug.fig);
            var disable_brush = brush_plug.disable.bind(brush_plug);
            brush_plug.fig.enable_zoom = function(){
                   disable_brush();
                   fig.toolbar.toolbar.selectAll(".mpld3-brushbutton")
                       .classed({pressed: false,
                                 active: false});
                   enable_zoom();
            };
            this.onClick();  // enable the button
        },
        icon: "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABmJLR0QA/wD/AP+gvaeTAAAACXBI
        \nWXMAAEQkAABEJAFAZ8RUAAAAB3RJTUUH3gMCEiQKB9YaAgAAAWtJREFUOMuN0r1qVVEQhuFn700k\nnfEvBq0iNiIiOKXgH4KCaBeIhWARK
        /EibLwFCwVLjyAWaQzRGG9grC3URkHUBKKgRuWohWvL5pjj\nyTSLxcz7rZlZHyMiItqzFxGTEVF18/UoODNFxDIO4x12dkXqTcBPsCUzD
        +AK3ndFqhHwEsYz82gn\nN4dbmMRK9R/4KY7jAvbiWmYeHBT5Z4QCP8J1rGAeN3GvU3Mbl/Gq3qCDcxjLzOV+v78fq/iFIxFx
        \nPyJ2lNJpfBy2g59YzMyzEbEVLzGBJjOriLiBq5gaJrCIU3hcRCbwAtuwjm/Yg/V6I9NgDA1OR8RC\nZq6Vcd7iUwtn5h8fdMBdETGPE
        +Xe4ExELDRNs4bX2NfCUHe+7UExyfkCP8MhzOA7PuAkvrbwXyNF
        \nxF3MDqxiqlhXC7SPdaOKiN14g0u4g3H0MvOiTUSNY3iemb0ywmfMdfYyUmAJ2yPiBx6Wr/oy2Oqw\n+A1SupBzAOuE
        /AAAAABJRU5ErkJggg==\n",
       });
    }

    LinkedBrushPlugin.prototype.onClick = function(){
      if(this.enabled){
        this.disable();
      }else{
        this.enable();
        this.fig.disable_zoom();
        this.fig.toolbar.toolbar.selectAll(".mpld3-movebutton")
                   .classed({pressed: false,
                             active: false});
      }
      this.fig.toolbar.toolbar.selectAll(".mpld3-brushbutton")
                .classed({pressed: this.enabled,
                          active: !this.enabled});
    }

    LinkedBrushPlugin.prototype.draw = function(){
      var obj = mpld3.get_element(this.prop.id);
      var fig = this.fig;
      var dataKey = ("offsets" in obj.prop) ? "offsets" : "data";

      mpld3.insert_css("#" + fig.figid + " rect.extent",
                       {"fill": "#000",
                        "fill-opacity": .125,
                        "stroke": "#fff"});

      mpld3.insert_css("#" + fig.figid + " path.mpld3-hidden",
                       {"stroke": "#ccc !important",
                        "fill": "#ccc !important"});

      var dataClass = "mpld3data-" + obj.prop[dataKey];

      var brush = d3.svg.brush()
                      .on("brushstart", brushstart)
                      .on("brush", brushmove)
                      .on("brushend", brushend);

      // Label all data points for access below
      fig.axes.forEach(function(ax){
         ax.elements.forEach(function(el){
            if(el.prop[dataKey] === obj.prop[dataKey]){
               el.group.classed(dataClass, true);
            }
         });
         brush.x(ax.x).y(ax.y);
         //ax.axes.call(brush);
      });


      // For fast brushing, precompute a list of selection properties
      // properties to apply to the selction.
      var data_map = [];
      var dataToBrush = fig.canvas.selectAll("." + dataClass)
                           .each(function(){
                              for(var i=0; i<fig.axes.length; i++){
                                var ax = fig.axes[i];
                                for(var j=0; j<ax.elements.length; j++){
                                  var el = ax.elements[j];
                                  if("group" in el && el.group[0][0] === this){
                                    data_map.push({i_ax: i,
                                                   ix: el.prop.xindex,
                                                   iy: el.prop.yindex});
                                    return;
                                  }
                                }
                              }
                            });

      dataToBrush.data(data_map)
                 .call(brush);

      var currentData;

      function brushstart(d){
        if(currentData != this){
          d3.select(currentData).call(brush.clear());
          currentData = this;
          brush.x(fig.axes[d.i_ax].x)
               .y(fig.axes[d.i_ax].y);
        }
      }

      function brushmove(d){
        var e = brush.extent();
        dataToBrush.selectAll("path")
                   .classed("mpld3-hidden",
                       function(p) {
                           return e[0][0] > p[d.ix] ||  e[1][0] < p[d.ix] ||
                                  e[0][1] > p[d.iy] || e[1][1] < p[d.iy];
                       });
      }

      function brushend(d){
        if (brush.empty()){
            dataToBrush.selectAll("path")
                       .classed("mpld3-hidden", false);
        }
      }

      function brushend_clear(){
        d3.select(this).call(brush.clear());
      }

      this.enable = function(){
        brush.on("brushstart", brushstart)
             .on("brush", brushmove)
             .on("brushend", brushend);
        this.fig.canvas.selectAll("rect.background")
              .style("cursor", "crosshair");
        this.fig.canvas.selectAll("rect.extent, rect.resize")
              .style("display", null);
        this.enabled = true;
      }

      this.disable = function(){
        brush.on("brushstart", null)
             .on("brush", null)
             .on("brushend", brushend_clear)
             .clear();
        this.fig.canvas.selectAll("rect.background")
              .style("cursor", null);
        this.fig.canvas.selectAll("rect.extent, rect.resize")
              .style("display", "none");
        this.enabled = false;
      }

      this.disable();
    }

    mpld3.register_plugin("linkedbrush", LinkedBrushPlugin);
    """

    def __init__(self, points):
        if isinstance(points, matplotlib.lines.Line2D):
            suffix = "pts"
        else:
            suffix = None

        self.dict_ = {"type": "linkedbrush",
                      "clear_toolbar": False,
                      "id": utils.get_id(points, suffix),
                      "buttons": "brush"}



__author__ = "David Rusk <drusk@uvic.ca>"

import sys

from ..astrom import AstromParser, StationaryParser
from ..downloads.async import AsynchronousDownloadManager
from ..downloads.cutouts.downloader import ImageCutoutDownloader
from ..gui import config, tasks, logger
from ..gui import context
from ..gui.controllers import (ProcessTracksController,
                               ProcessRealsController,
                               ProcessCandidatesController, ProcessVettingController, ProcessExamineController)
from ..gui.errorhandling import DownloadErrorHandler
from ..gui.models.imagemanager import ImageManager
from ..gui.models.transactions import TransAckValidationModel
from ..gui.models.workload import (WorkUnitProvider,
                                   RealsWorkUnitBuilder,
                                   TracksWorkUnitBuilder,
                                   CandidatesWorkUnitBuilder,
                                   VettingWorkUnitBuilder,
                                   ExamineWorkUnitBuilder,
                                   PreFetchingWorkUnitProvider)
from ..gui.sync import SynchronizationManager
from ..gui.views.appview import ApplicationView
from ..naming import ProvisionalNameGenerator, DryRunNameGenerator
from ..ssos import TracksParser, TrackTarget


def create_application(task_name, working_directory, output_directory,
                       dry_run=False, debug=False, name_filter=None, user_id=None,
                       skip_previous=False, zoom=1):
    logger.info("Starting %s task." % task_name)

    if task_name == tasks.CANDS_TASK:
        ProcessCandidatesApplication(working_directory, output_directory,
                                     dry_run=dry_run, debug=debug, name_filter=name_filter, user_id=user_id, zoom=zoom)
    elif task_name == tasks.REALS_TASK:
        ProcessRealsApplication(working_directory, output_directory,
                                dry_run=dry_run, debug=debug, name_filter=name_filter, user_id=user_id, zoom=zoom)
    elif task_name  == tasks.VETTING_TASK:
        ProcessVettingApplication(working_directory, output_directory, dry_run=dry_run,
                                     debug=debug, name_filter=name_filter, user_id=user_id, zoom=zoom)
    elif task_name == tasks.EXAMINE_TASK:
        ProcessExamineApplication(working_directory, output_directory, dry_run=dry_run,
                                  debug=debug, name_filter=name_filter, user_id=user_id, zoom=zoom)
    elif task_name == tasks.TRACK_TASK:
        ProcessTracksApplication(working_directory, output_directory,
                                 dry_run=dry_run, debug=debug, name_filter=name_filter,
                                 skip_previous=skip_previous, user_id=user_id, zoom=zoom)
    elif task_name == tasks.TARGET_TASK:
        ProcessTargetApplication(working_directory, output_directory,
                                 dry_run=dry_run, debug=debug, name_filter=name_filter,
                                 skip_previous=skip_previous, user_id=user_id, zoom=zoom)
    else:
        error_message = "Unknown task: %s" % task_name
        logger.critical(error_message)
        raise ValueError(error_message)


class ValidationApplication(object):
    def __init__(self, working_directory, output_directory,
                 dry_run=False, debug=False, name_filter=None, user_id=None, mark_using_pixels=True, zoom=1):

        self.dry_run = dry_run
        self.user_id = user_id
        logger.info("Input directory set to: %s" % working_directory)
        logger.info("Output directory set to: %s" % output_directory)

        working_context = context.get_context(working_directory, userid=self.user_id)
        output_context = context.get_context(output_directory, userid=self.user_id)

        if dry_run and working_context.is_remote():
            sys.stdout.write("A dry run can only be done on local files.\n")
            sys.exit(0)

        if output_context.is_remote():
            sys.stdout.write("The output directory must be local.\n")
            sys.exit(0)

        image_manager = self._create_image_manager()

        progress_manager = working_context.get_progress_manager()
        builder = self._create_workunit_builder(working_context,
                                                output_context,
                                                progress_manager)

        workunit_provider = WorkUnitProvider(self.input_suffix,
                                             working_context,
                                             progress_manager, builder,
                                             randomize=self.should_randomize_workunits,
                                             name_filter=name_filter)

        prefetching_workunit_provider = PreFetchingWorkUnitProvider(workunit_provider,
                                                                    config.read("PREFETCH.NUMBER"),
                                                                    image_manager)

        if working_context.is_remote():
            synchronization_manager = SynchronizationManager(working_context, sync_enabled=True)
        else:
            synchronization_manager = None

        model = TransAckValidationModel(prefetching_workunit_provider,
                                        image_manager,
                                        synchronization_manager)
        logger.debug("Created model.")

        view = self._create_view(model, debug=debug, mark_using_pixels=mark_using_pixels, zoom=zoom)

        logger.debug("Created view.")
        model.start_work()

        self.model = model
        self.view = view
        self.controller = view.controller

        self.controller.display_current_image()

        if not synchronization_manager:
            self.view.disable_sync_menu()

        self.view.show()

    def _create_image_manager(self):
        error_handler = DownloadErrorHandler(self)

        def read(slice_config):
            return config.read("CUTOUTS.%s" % slice_config)

        singlet_downloader = ImageCutoutDownloader(
            slice_rows=read("SINGLETS.SLICE_ROWS"),
            slice_cols=read("SINGLETS.SLICE_COLS"))

        singlet_download_manager = AsynchronousDownloadManager(
            singlet_downloader, error_handler)

        triplet_downloader = ImageCutoutDownloader(
            slice_rows=read("TRIPLETS.SLICE_ROWS"),
            slice_cols=read("TRIPLETS.SLICE_COLS"))

        triplet_download_manager = AsynchronousDownloadManager(
            triplet_downloader, error_handler)

        return ImageManager(singlet_download_manager, triplet_download_manager)

    def get_model(self):
        return self.model

    def get_view(self):
        return self.view

    @property
    def input_suffix(self):
        raise NotImplementedError()

    @property
    def should_randomize_workunits(self):
        raise NotImplementedError()

    def _create_view(self, model, debug=False, mark_using_pixels=False, zoom=1):
        return ApplicationView(self._create_controller_factory(model),
                               debug=debug,
                               mark_using_pixels=mark_using_pixels, zoom=zoom)

    def _create_workunit_builder(self,
                                 input_context,
                                 output_context,
                                 progress_manager):
        raise NotImplementedError()

    def _create_controller_factory(self, model):
        raise NotImplementedError()


class ProcessCandidatesApplication(ValidationApplication):
    @property
    def input_suffix(self):
        return tasks.suffixes[tasks.CANDS_TASK]

    @property
    def should_randomize_workunits(self):
        return True

    def _create_workunit_builder(self,
                                 input_context,
                                 output_context,
                                 progress_manager):
        return CandidatesWorkUnitBuilder(
            AstromParser(), input_context, output_context, progress_manager,
            dry_run=self.dry_run)

    def _create_controller_factory(self, model):
        return CandidatesControllerFactory(model, dry_run=self.dry_run)


class ProcessRealsApplication(ValidationApplication):
    def __init__(self, working_directory, output_directory,
                 dry_run=False, debug=False, name_filter=None, user_id=None, zoom=1):
        preload_iraf()

        super(ProcessRealsApplication, self).__init__(
            working_directory, output_directory, dry_run=dry_run,
            debug=debug, name_filter=name_filter, user_id=user_id, mark_using_pixels=False, zoom=zoom)

    @property
    def input_suffix(self):
        return tasks.suffixes[tasks.REALS_TASK]

    @property
    def should_randomize_workunits(self):
        return True  # now we are only going to manually measure the genuine candidates not also the planted ones

    def _create_workunit_builder(self,
                                 input_context,
                                 output_context,
                                 progress_manager):
        return RealsWorkUnitBuilder(
            AstromParser(), input_context, output_context, progress_manager,
            dry_run=self.dry_run)

    def _create_controller_factory(self, model):
        return RealsControllerFactory(model, dry_run=self.dry_run)


class ProcessVettingApplication(ProcessCandidatesApplication):

    @property
    def input_suffix(self):
        return tasks.suffixes[tasks.VETTING_TASK]

    @property
    def should_randomize_workunits(self):
        return False  # now we are only going to manually measure the genuine candidates not also the planted ones

    def _create_controller_factory(self, model):
        return VettingControllerFactory(model, dry_run=self.dry_run)

    def _create_workunit_builder(self,
                                 input_context,
                                 output_context,
                                 progress_manager):
        return VettingWorkUnitBuilder(StationaryParser(), input_context, output_context, progress_manager,
                                      dry_run=self.dry_run)

class ProcessExamineApplication(ProcessRealsApplication):

    @property
    def input_suffix(self):
        return tasks.suffixes[tasks.EXAMINE_TASK]

    @property
    def should_randomize_workunits(self):
        return False  # now we are only going to manually measure the genuine candidates not also the planted ones

    def _create_controller_factory(self, model):
        return ExamineControllerFactory(model, dry_run=self.dry_run)

    def _create_workunit_builder(self,
                                 input_context,
                                 output_context,
                                 progress_manager):
        return ExamineWorkUnitBuilder(StationaryParser(discovery_only=False),
                                      input_context, output_context,
                                      progress_manager,
                                      dry_run=self.dry_run)



class ProcessTracksApplication(ValidationApplication):
    def __init__(self, working_directory, output_directory,
                 dry_run=False, debug=False, name_filter=None, skip_previous=False,
                 user_id=None, zoom=1):
        preload_iraf()
        self.skip_previous = skip_previous
        super(ProcessTracksApplication, self).__init__(
            working_directory, output_directory, dry_run=dry_run, debug=debug, 
            name_filter=name_filter,
            user_id=user_id, zoom=zoom)
        self.controller.is_discovery = False

    @property
    def input_suffix(self):
        return tasks.suffixes[tasks.TRACK_TASK]

    @property
    def should_randomize_workunits(self):
        return False

    def _create_workunit_builder(self,
                                 input_context,
                                 output_context,
                                 progress_manager):
        return TracksWorkUnitBuilder(
            TracksParser(skip_previous=self.skip_previous), input_context, output_context, progress_manager,
            dry_run=self.dry_run)

    def _create_controller_factory(self, model):
        return TracksControllerFactory(model, dry_run=self.dry_run)

    def _create_view(self, model, debug=False, mark_using_pixels=False, zoom=1):
        return ApplicationView(self._create_controller_factory(model),
                               track_mode=True, debug=debug, zoom=zoom)


class ProcessTargetApplication(ProcessTracksApplication):
    def __init__(self, working_directory, output_directory,
                 dry_run=False, debug=False, name_filter=None, skip_previous=False,
                 user_id=None, zoom=1):
        preload_iraf()
        self.skip_previous = skip_previous
        super(ProcessTargetApplication, self).__init__(
            working_directory, output_directory, dry_run=dry_run, debug=debug,
            name_filter=name_filter,
            user_id=user_id, zoom=zoom)
        self.controller.is_discovery = False

    @property
    def input_suffix(self):
        return tasks.suffixes[tasks.TARGET_TASK]

    @property
    def should_randomize_workunits(self):
        return False

    def _create_workunit_builder(self,
                                 input_context,
                                 output_context,
                                 progress_manager):
        return TracksWorkUnitBuilder(
            TrackTarget(skip_previous=self.skip_previous), input_context, output_context, progress_manager,
            dry_run=self.dry_run)

    def _create_controller_factory(self, model):
        return TracksControllerFactory(model, dry_run=self.dry_run)

    def _create_view(self, model, debug=False, mark_using_pixels=False, zoom=1):
        return ApplicationView(self._create_controller_factory(model),
                               track_mode=True, debug=debug, zoom=zoom)


class ControllerFactory(object):
    """
    Allows the view to create the controller without direct knowledge
    of the model.
    """

    def __init__(self, model, dry_run=False):
        self.model = model
        self.dry_run = dry_run

    def create_controller(self, view):
        raise NotImplementedError()


class CandidatesControllerFactory(ControllerFactory):
    def create_controller(self, view):
        return ProcessCandidatesController(self.model, view)


class VettingControllerFactory(ControllerFactory):
    def create_controller(self, view):
        return ProcessVettingController(self.model, view)


class ExamineControllerFactory(ControllerFactory):

    def create_controller(self, view):

        return ProcessExamineController(self.model, view, name_generator=ProvisionalNameGenerator())


class RealsControllerFactory(ControllerFactory):
    def create_controller(self, view):
        if self.dry_run:
            name_generator = DryRunNameGenerator()
        else:
            name_generator = ProvisionalNameGenerator()

        return ProcessRealsController(self.model, view, name_generator)


class TracksControllerFactory(ControllerFactory):
    def create_controller(self, view):
        if self.dry_run:
            name_generator = DryRunNameGenerator()
        else:
            name_generator = ProvisionalNameGenerator()

        return ProcessTracksController(self.model, view, name_generator)


def preload_iraf():
    logger.info("Preloading IRAF")
    # NOTE: Force expensive loading of libraries up front.  These are
    # libraries that the reals task needs but the candidates task
    # doesn't.  To make sure the candidates task doesn't load them, we
    # import them directly in the functions/methods where they are used.
    # TODO: find out what the best practice is for handling this sort of
    # situation and refactor.


__author__ = 'jjk'
import unittest
import sys

from hamcrest import assert_that, equal_to, has_length, contains

from astropy import coordinates
from astropy import units

from ossos import mpc
from ossos import orbfit
import os


class OrbfitTest(unittest.TestCase):

    def test_orbfit_residuals(self):
        mpc_lines=("     HL7j2    C2013 04 03.62926 17 12 01.16 +04 13 33.3          24.1 R      568",
                   "     HL7j2    C2013 04 04.58296 17 11 59.80 +04 14 05.5          24.0 R      568",
                   "     HL7j2    C2013 05 03.52252 17 10 38.28 +04 28 00.9          23.4 R      568",
                   "     HL7j2    C2013 05 08.56725 17 10 17.39 +04 29 47.8          23.4 R      568")

        observations = []
        for line in mpc_lines:
            observations.append(mpc.Observation.from_string(line))

        HL7j2 = orbfit.Orbfit(observations=observations)

        for observation in observations:
            HL7j2.predict(observation.date, 568)
            self.assertLess(observation.ra_residual, 0.3)
            self.assertLess(observation.dec_residual, 0.3)

    def test_null_obseravtion(self):
        mpc_lines=("!    HL7j2    C2013 04 03.62926 17 12 01.16 +04 13 33.3          24.1 R      568",
                   "     HL7j2    C2013 04 04.58296 17 11 59.80 +04 14 05.5          24.0 R      568",
                   "     HL7j2    C2013 05 03.52252 17 10 38.28 +04 28 00.9          23.4 R      568",
                   "     HL7j2    C2013 05 08.56725 17 10 17.39 +04 29 47.8          23.4 R      568")

        observations  = []
        for line in mpc_lines:
            observations.append(mpc.Observation.from_string(line))

        HL7j2 = orbfit.Orbfit(observations)
        self.assertAlmostEqual(HL7j2.a, 135.75, 1)


if __name__ == '__main__':
    unittest.main()





import logging
import util
from astropy.table import Table, Column
import numpy
import wcs
import storage



VOS_SCHEME = 'vos'
FILE_SCHEME = 'file'
MJD_OFFSET = 2400000.5

class Parser(object):
    """Read in a MOP formatted file"""

    def __init__(self, expnum, ccd, extension, type='p', prefix=None):
        """does nothing"""

        self.expnum = expnum
        self.ccd = ccd
        self.extension = extension
        self.type = type
        self.prefix = prefix
        self.fobj = None
        self.keywords  = []
        self.values = []
        self.formats = {}
        self.header = {}
        self.cdata ={}
        

    def parse(self):
        """read in a file and return a MOPFile object."""
        self.filename =  storage.get_file(self.expnum,
                                       self.ccd,
                                       ext=self.extension,
                                       version=self.type,
                                       prefix=self.prefix)
        self.fobj = open(self.filename,'r')
        lines = self.fobj.read().split('\n')
        self.header = HeaderParser(self.extension).parser(lines)
        if 'matt' in self.extension:
            usecols=[0,1,2,3,4]
            data = numpy.genfromtxt(self.filename, usecols=usecols)
        else:
            data = numpy.genfromtxt(self.filename)
        self.data = Table(data, names=self.header.column_names[0:data.shape[1]])
        ast_header = storage.get_astheader(self.expnum, self.ccd)
        self.wcs = wcs.WCS(ast_header)
        flip_these_extensions = range(1,19)
        flip_these_extensions.append(37)
        flip_these_extensions.append(38)
        if self.ccd + 1 in flip_these_extensions:
            self.data['X'] = float(self.header.keywords['NAX1'][0])-self.data['X'] + 1
            self.data['Y'] = float(self.header.keywords['NAX2'][0])-self.data['Y'] + 1
        ra, dec = self.wcs.xy2sky(self.data['X'], self.data['Y'], usepv=True)

        self.data.add_columns([Column(ra, name='RA_J2000'), Column(dec, name='DE_J2000')])
        return self



        
class HeaderParser(object):
    """A MOPFile Header object"""

    def __init__(self, extension):

        self.keywords = {}
        self.extension = extension
        self.file_ids = []
        self.column_names = []

    def __str__(self):
        return str(self.keywords)+'\n'+str(self.column_names)

    def parser(self, lines):
        """Given a set of lines parse the into a MOP Header"""
        while len(lines) > 0:
            if lines[0].startswith('##') and lines[1].startswith('# '):
                ## A keyword line/value pair starts here
                self._header_append(lines.pop(0),lines.pop(0))
            elif lines[0].startswith('# '):
                ## Filenames start here
                self._append_file_id(lines.pop(0))
            elif lines[0].startswith('##'):
                self._set_column_names(lines.pop(0)[2:], ext= self.extension)
            else:
                return self
        raise IOError("Failed trying to read header")

    def _compute_mjd(self, kw, val):
        """
        Sometimes that MJD-OBS-CENTER keyword maps to a three component string, instead of a single value.
        @param kw: a list of keywords
        @param val: a list of matching values
        @return: the MJD-OBS-CENTER as a julian date.
        """
        try:
            idx = kw.index('MJD-OBS-CENTER')
        except ValueError:
            return
        if len(val) == len(kw):
            return
        if len(val) - 2 != len(kw):
            raise ValueError("convert: keyword/value lengths don't match: {}/{}".format(kw, val))
        val.insert(idx, util.Time("{} {} {}".format(val.pop(idx), val.pop(idx), val.pop(idx)),
                             format='mpc',
                             scale='utc').mjd)
        logging.debug("Computed MJD: {}".format(val[idx]))

    def _header_append(self, kw_line, val_line):
        
        keywords = kw_line.strip().split()
        values = val_line.strip().split()
        self._compute_mjd(keywords, values)
        self._append_keywords(keywords, values)
        return

    def _depreicated_compute_mjd(self, year, month, day):
        """Look through the dictionary and create a new keyword list
        by replacing mjd-obs-center keyword value with actual mjd"""
        mjd = util.Time("{} {} {}".format(year, month, day), format='mpc', scale='utc').mjd
        return mjd

    def _append_keywords(self, keywords, values):

        if len(keywords) != len(values):
            raise ValueError("keyword/value lengths don't match: {}/{}".format(keywords, values))
        while len(keywords) > 0:
            keyword = keywords.pop()
            value = values.pop()
            if not self.keywords.has_key(keyword) :
                self.keywords[keyword] = []
            self.keywords[keyword].append(value)
        return

    def _append_file_id(self, line):
        self.file_ids.append(line.split()[1].strip())


    def _set_column_names(self, line, ext='obj.jmp'):
        if 'matt' in ext:
            self.column_names = line[1:].split()[0:5]
        else:
            self.column_names = line[1:].split()
        print ext, len(self.column_names), line



from copy import deepcopy
import warnings

from astropy.units import Quantity

__author__ = "David Rusk <drusk@uvic.ca>"

import math
from astropy import wcs as astropy_wcs
from astropy import units
import numpy
from gui import logger


PI180 = 57.2957795130823208767981548141052


class WCS(astropy_wcs.WCS):
    def __init__(self, header):
        """
        Create the bits needed for working with sky2xy
        """
        astropy_header = deepcopy(header)
        del (astropy_header['PV*'])
        with warnings.catch_warnings(record=True):
            warnings.resetwarnings()
            warnings.simplefilter(
                "ignore", astropy_wcs.FITSFixedWarning, append=True)
            super(WCS, self).__init__(astropy_header)
        self.header = header

    @property
    def cd(self):
        """
        CD Rotation matrix values.
        """
        return parse_cd(self.header)

    @property
    def dc(self):
        """
        CD Rotation matrix INVERTED i.e.  []^-1
        """

        return numpy.array(numpy.mat(self.cd).I)

    @property
    def pv(self):
        """
        Array of PV keywords used for hi-odered astrogwyn mapping
        """
        return parse_pv(self.header)

    @property
    def crpix1(self):
        """
        1st reference coordinate
        """
        return self.header['CRPIX1']

    @property
    def crpix2(self):
        """
        2nd reference coordinate
        """
        return self.header['CRPIX2']

    @property
    def crval1(self):
        """
        Reference Coordinate of 1st reference pixel
        """
        return self.header['CRVAL1']

    @property
    def crval2(self):
        """
        Reference Coordinate of 2nd reference pixel
        """
        return self.header['CRVAL2']

    @property
    def nord(self):
        """
        The order of the PV fit, provided by astgwyn
        """
        return self.header['NORDFIT']

    def xy2sky(self, x, y, usepv=True):
        if usepv:
            try:
                return xy2skypv(x=numpy.array(x), y=numpy.array(y),
                                crpix1=self.crpix1,
                                crpix2=self.crpix2,
                                crval1=self.crval1,
                                crval2=self.crval2,
                                cd=self.cd,
                                pv=self.pv,
                                nord=self.nord)
            except Exception as ex:
                logger.warning("Error {} {}".format(type(ex), ex))
                logger.warning("Reverted to CD-Matrix WCS.")
                print ex
        xy = numpy.array([x, y]).transpose()
        pos = self.wcs_pix2world(xy, 1).transpose()
        return pos[0] * units.degree, pos[1] * units.degree

    def sky2xy(self, ra, dec, usepv=True):
        if isinstance(ra, Quantity):
            ra = ra.to(units.degree).value
        if isinstance(dec, Quantity):
            dec = dec.to(units.degree).value
        try:
            if usepv:
                return sky2xypv(ra=ra,
                                dec=dec,
                                crpix1=self.crpix1,
                                crpix2=self.crpix2,
                                crval1=self.crval1,
                                crval2=self.crval2,
                                dc=self.dc,
                                pv=self.pv,
                                nord=self.nord)
        except Exception as ex:
            logger.warning("sky2xy raised exception: {0}".format(ex))
            logger.warning("Reverted to CD-Matrix WCS to convert: {0} {1} ".format(ra, dec))
            print ex
        pos = self.wcs_world2pix([[ra, dec], ], 1)
        return pos[0][0], pos[0][1]


def sky2xypv(ra, dec, crpix1, crpix2, crval1, crval2, dc, pv, nord, maxiter=300):
    """
    Transforms from celestial coordinates to pixel coordinates to taking
    non-linear distortion into account with the World Coordinate System
    FITS keywords as used in MegaPipe.

    For the inverse operation see xy2sky.

    Reference material:
    http://www.cadc-ccda.hia-iha.nrc-cnrc.gc.ca/megapipe/docs/CD_PV_keywords.pdf

    Args:
      ra: float
        Right ascension
      dec: float
        Declination
      crpix1: float
        Tangent point x, pixels
      crpix2: float
        Tangent point y, pixels
      crval1: float
        Tangent point RA, degrees
      crval2:
        Tangent point Dec, degrees
      dc: 2d array
        Expresses the scale, the rotation and any possible skew of the image
        with respect to the sky.  It is the inverse of the cd matrix in xy2sky.
      pv: 2d array
      nord: int
        order of the fit

    Returns:
      x, y: float
        Pixel coordinates
    """
    if math.fabs(ra - crval1) > 100:
        if crval1 < 180:
            ra -= 360
        else:
            ra += 360

    ra /= PI180
    dec /= PI180

    tdec = math.tan(dec)
    ra0 = crval1 / PI180
    dec0 = crval2 / PI180
    ctan = math.tan(dec0)
    ccos = math.cos(dec0)

    traoff = math.tan(ra - ra0)
    craoff = math.cos(ra - ra0)
    etar = (1 - ctan * craoff / tdec) / (ctan + craoff / tdec)
    xir = traoff * ccos * (1 - etar * ctan)
    xi = xir * PI180
    eta = etar * PI180

    if nord < 0:
        # The simple solution
        x = xi
        y = eta
    else:
        # Reverse by Newton's method
        tolerance = 0.001 / 3600

        # Initial guess
        x = xi
        y = eta
        iteration = 0

        converged = False
        while not converged:
            assert nord >= 0

            # Estimates
            f = pv[0][0]
            g = pv[1][0]

            # Derivatives
            fx = 0
            fy = 0
            gx = 0
            gy = 0

            if nord >= 1:
                r = math.sqrt(x ** 2 + y ** 2)
                f += pv[0][1] * x + pv[0][2] * y + pv[0][3] * r
                g += pv[1][1] * y + pv[1][2] * x + pv[1][3] * r
                fx += pv[0][1] + pv[0][3] * x / r
                fy += pv[0][2] + pv[0][3] * y / r
                gx += pv[1][2] + pv[1][3] * x / r
                gy += pv[1][1] + pv[1][3] * y / r

            if nord >= 2:
                x2 = x ** 2
                xy = x * y
                y2 = y ** 2

                f += pv[0][4] * x2 + pv[0][5] * xy + pv[0][6] * y2
                g += pv[1][4] * y2 + pv[1][5] * xy + pv[1][6] * x2
                fx += pv[0][4] * 2 * x + pv[0][5] * y
                fy += pv[0][5] * x + pv[0][6] * 2 * y
                gx += pv[1][5] * y + pv[1][6] * 2 * x
                gy += pv[1][4] * 2 * y + pv[1][5] * x

            if nord >= 3:
                x3 = x ** 3
                x2y = x2 * y
                xy2 = x * y2
                y3 = y ** 3

                f += pv[0][7] * x3 + pv[0][8] * x2y + pv[0][9] * xy2 + pv[0][10] * y3
                g += pv[1][7] * y3 + pv[1][8] * xy2 + pv[1][9] * x2y + pv[1][10] * x3
                fx += pv[0][7] * 3 * x2 + pv[0][8] * 2 * xy + pv[0][9] * y2
                fy += pv[0][8] * x2 + pv[0][9] * 2 * xy + pv[0][10] * 3 * y2
                gx += pv[0][8] * y2 + pv[1][9] * 2 * xy + pv[1][10] * 3 * x2
                gy += pv[1][7] * 3 * y2 + pv[0][8] * 2 * xy + pv[1][9] * x2

            f -= xi
            g -= eta
            dx = (-f * gy + g * fy) / (fx * gy - fy * gx)
            dy = (-g * fx + f * gx) / (fx * gy - fy * gx)
            x += dx
            y += dy

            if math.fabs(dx) < tolerance and math.fabs(dy) < tolerance:
                converged = True

            iteration += 1

            if iteration > maxiter:
                break

    xp = dc[0][0] * x + dc[0][1] * y
    yp = dc[1][0] * x + dc[1][1] * y

    x = xp + crpix1
    y = yp + crpix2

    return x, y


def xy2skypv(x, y, crpix1, crpix2, crval1, crval2, cd, pv, nord):
    """
    Transforms from pixel coordinates to celestial coordinates taking
    non-linear distortion into account with the World Coordinate System
    FITS keywords as used in MegaPipe.

    For the inverse operation see sky2xy

    Reference material:
    http://www.cadc-ccda.hia-iha.nrc-cnrc.gc.ca/megapipe/docs/CD_PV_keywords.pdf

    Args:
      x, y: float
        Input pixel coordinate
      crpix1: float
        Tangent point x, pixels
      crpix2: float
        Tangent point y, pixels
      crval1: float
        Tangent point RA, degrees
      crval2:
        Tangent point Dec, degrees
      cd: 2d array
        Expresses the scale, the rotation and any possible skew of the image
        with respect to the sky.
      pv: 2d array
      nord: int
        order of the fit

    Returns:
      ra: float
        Right ascension
      dec: float
        Declination
    """
    xp = x - crpix1
    yp = y - crpix2

    # IMPORTANT NOTE: 0-based indexing in Python means indexing for values
    # in cd and pv will be shifted from in the paper.
    x_deg = cd[0][0] * xp + cd[0][1] * yp
    y_deg = cd[1][0] * xp + cd[1][1] * yp

    if nord < 0:
        xi = x
        eta = y
    else:
        xi = pv[0][0]
        eta = pv[1][0]

    if nord >= 1:
        r = numpy.sqrt(x_deg ** 2 + y_deg ** 2)
        xi += pv[0][1] * x_deg + pv[0][2] * y_deg + pv[0][3] * r
        eta += pv[1][1] * y_deg + pv[1][2] * x_deg + pv[1][3] * r

        if nord >= 2:
            x2 = x_deg ** 2
            xy = x_deg * y_deg
            y2 = y_deg ** 2

            xi += pv[0][4] * x2 + pv[0][5] * xy + pv[0][6] * y2
            eta += pv[1][4] * y2 + pv[1][5] * xy + pv[1][6] * x2

            if nord >= 3:
                x3 = x_deg ** 3
                x2y = x2 * y_deg
                xy2 = x_deg * y2
                y3 = y_deg ** 3

                xi += pv[0][7] * x3 + pv[0][8] * x2y + pv[0][9] * xy2 + pv[0][10] * y3
                eta += pv[1][7] * y3 + pv[1][8] * xy2 + pv[1][9] * x2y + pv[1][10] * x3

    xir = xi / PI180
    etar = eta / PI180

    ra0 = crval1 / PI180
    dec0 = crval2 / PI180

    ctan = numpy.tan(dec0)
    ccos = numpy.cos(dec0)
    raoff = numpy.arctan2(xir / ccos, 1 - etar * ctan)
    ra = raoff + ra0
    dec = numpy.arctan(numpy.cos(raoff) / ((1 - (etar * ctan)) / (etar + ctan)))

    ra *= PI180
    ra = numpy.where(ra < 0 , ra + 360, ra)
    ra = numpy.where(ra > 360, ra - 360, ra)
    # f = numpy.int(ra < 0) + 360
    # print f
    # if ra < 0:
    #     ra += 360
    # if ra >= 360:
    #     ra -= 360

    dec *= PI180

    return ra * units.degree, dec * units.degree


def parse_cd(header):
    """
    Parses the CD array from an astropy FITS header.

    Args:
      header: astropy.io.fits.header.Header
        The header containing the CD values.

    Returns:
      cd: 2d array (list(list(float))
        [[CD1_1, CD1_2], [CD2_1, CD2_2]]
    """
    return [[header["CD1_1"], header["CD1_2"]],
            [header["CD2_1"], header["CD2_2"]]]


def parse_pv(header):
    """
    Parses the PV array from an astropy FITS header.

    Args:
      header: astropy.io.fits.header.Header
        The header containing the PV values.

    Returns:
      cd: 2d array (list(list(float))
        [[PV1_0, PV1_1, ... PV1_N], [PV2_0, PV2_1, ... PV2_N]]
        Note that N depends on the order of the fit.  For example, an
        order 3 fit goes up to PV?_10.
    """
    order_fit = parse_order_fit(header)

    def parse_with_base(i):
        key_base = "PV%d_" % i

        pvi_x = [header[key_base + "0"]]

        def parse_range(lower, upper):
            for j in range(lower, upper + 1):
                pvi_x.append(header[key_base + str(j)])

        if order_fit >= 1:
            parse_range(1, 3)

        if order_fit >= 2:
            parse_range(4, 6)

        if order_fit >= 3:
            parse_range(7, 10)

        return pvi_x

    return [parse_with_base(1), parse_with_base(2)]


def parse_order_fit(header):
    """
    Parses the order of the fit for PV terms.

    Args:
      header: astropy.io.fits.header.Header
        The header containing the PV values.

    Returns:
      order_fit: int
        The order of the fit.
    """
    return int(header["NORDFIT"])



import json
import logging
import os
import math

from astropy import wcs
from astropy.io import ascii
from astropy.io import fits
import numpy

import storage
import util


def get_wcs(shifts):
    # store the shifts as a WCS transform where the 'world' coordinates are really the pixel coordinates
    # in the reference frame.
    return wcs.WCS(header=shifts, naxis=2)


def align(expnums, ccd, version='s', dry_run=False):
    """Create a 'shifts' file that transforms the space/flux/time scale of all images to the first image.

    This function relies on the .fwhm, .trans.jmp, .phot and .zeropoint.used files for inputs.
    The scaling we are computing here is for use in planting sources into the image at the same sky/flux locations
    while accounting for motions of sources with time.

    :param expnums: list of MegaPrime exposure numbers to add artificial KBOs to,
                    the first frame in the list is the reference.
    :param ccd: which ccd to work on.
    :param version: Add sources to the 'o', 'p' or 's' images
    :param dry_run: don't push results to VOSpace.
    """

    # Get the images and supporting files that we need from the VOSpace area
    # get_image and get_file check if the image/file is already on disk.
    # re-computed fluxes from the PSF stars and then recompute x/y/flux scaling.

    # some dictionaries to hold the various scale
    pos = {}
    apcor = {}
    mags = {}
    zmag = {}
    mjdates = {}

    for expnum in expnums:
        filename = storage.get_image(expnum, ccd=ccd, version=version)
        zmag[expnum] = storage.get_zeropoint(expnum, ccd, prefix=None, version=version)
        mjdates[expnum] = float(fits.open(filename)[0].header.get('MJD-OBS'))
        apcor[expnum] = [float(x) for x in open(storage.get_file(expnum,
                                                                 ccd=ccd,
                                                                 version=version,
                                                                 ext=storage.APCOR_EXT)).read().split()]
        keys = ['crval1', 'cd1_1', 'cd1_2', 'crval2', 'cd2_1', 'cd2_2']
        # load the .trans.jmp values into a 'wcs' like dictionary.
        # .trans.jmp maps current frame to reference frame in pixel coordinates.
        # the reference frame of all the frames supplied must be the same.
        shifts = dict(zip(keys, [float(x) for x in open(storage.get_file(expnum,
                                                                         ccd=ccd,
                                                                         version=version,
                                                                         ext='trans.jmp')).read().split()]))
        shifts['crpix1'] = 0.0
        shifts['crpix2'] = 0.0
        # now create a wcs object based on those transforms, this wcs links the current frame's
        # pixel coordinates to the reference frame's pixel coordinates.
        w = get_wcs(shifts)

        # get the PHOT file that was produced by the mkpsf routine
        logging.debug("Reading .phot file {}".format(expnum))
        phot = ascii.read(storage.get_file(expnum, ccd=ccd, version=version, ext='phot'), format='daophot')

        # compute the small-aperture magnitudes of the stars used in the PSF
        import daophot
        logging.debug("Running phot on {}".format(filename))
        mags[expnum] = daophot.phot(filename,
                                    phot['XCENTER'],
                                    phot['YCENTER'],
                                    aperture=apcor[expnum][0],
                                    sky=apcor[expnum][1] + 1,
                                    swidth=apcor[expnum][0],
                                    zmag=zmag[expnum])

        # covert the x/y positions to positions in Frame 1 based on the trans.jmp values.
        logging.debug("Doing the XY translation to refrence frame: {}".format(w))
        (x, y) = w.wcs_pix2world(mags[expnum]["XCENTER"], mags[expnum]["YCENTER"], 1)
        pos[expnum] = numpy.transpose([x, y])
        # match this exposures PSF stars position against those in the first image of the set.
        logging.debug("Matching lists")
        idx1, idx2 = util.match_lists(pos[expnums[0]], pos[expnum])

        # compute the magnitdue offset between the current frame and the reference.
        dmags = numpy.ma.array(mags[expnums[0]]["MAG"] - apcor[expnums[0]][2] -
                               (mags[expnum]["MAG"][idx1] - apcor[expnum][2]),
                               mask=idx1.mask)
        dmags.sort()
        logging.debug("Computed dmags between input and reference: {}".format(dmags))
        error_count = 0

        error_count += 1
        logging.debug("{}".format(error_count))

        # compute the median and determine if that shift is small compared to the scatter.
        try:
            midx = int(numpy.sum(numpy.any([~dmags.mask], axis=0)) / 2.0)
            dmag = float(dmags[midx])
            logging.debug("Computed a mag delta of: {}".format(dmag))
        except Exception as e:
            logging.error(str(e))
            logging.error("Failed to compute mag offset between plant and found using: {}".format(dmags))
            dmag = 99.99

        error_count += 1
        logging.debug("{}".format(error_count))

        try:
            if math.fabs(dmag) > 3 * (dmags.std() + 0.01):
                logging.warning("Magnitude shift {} between {} and {} is large: {}".format(dmag,
                                                                                           expnums[0],
                                                                                           expnum,
                                                                                           shifts))
        except Exception as e:
            logging.error(str(e))

        error_count += 1
        logging.debug("{}".format(error_count))

        shifts['dmag'] = dmag
        shifts['emag'] = dmags.std()
        shifts['nmag'] = len(dmags.mask) - dmags.mask.sum()
        shifts['dmjd'] = mjdates[expnums[0]] - mjdates[expnum]
        shift_file = os.path.basename(storage.get_uri(expnum, ccd, version, '.shifts'))

        error_count += 1
        logging.debug("{}".format(error_count))

        try:
            fh = open(shift_file, 'w')
            fh.write(json.dumps(shifts, sort_keys=True, indent=4, separators=(',', ': ')))
            fh.write('\n')
            fh.close()
        except Exception as e:
            logging.error("Creation of SHIFTS file failed while trying to write: {}".format(shifts))
            raise e

        error_count += 1
        logging.debug("{}".format(error_count))

        if not dry_run:
            storage.copy(shift_file, storage.get_uri(expnum, ccd, version, '.shifts'))



from unittest import TestCase
from astropy import coordinates
from astropy.time import Time
from astropy import units
import cStringIO

__author__ = 'jjk'

from ossos import ephem_target

example_file="""<?xml version="1.0" ?>
<!DOCTYPE ASTRO
  SYSTEM 'http://vizier.u-strasbg.fr/xml/astrores.dtd'>
  <ASTRO ID="v0.8" xmlns:ASTRO="http://vizier.u-strasbg.fr/doc/astrores.htx">
    <TABLE ID="Table">
      <NAME>Ephemeris</NAME>
      <TITLE>Ephemeris for CFHT QSO target Test</TITLE>
      <!--Definition of each field-->
      <FIELD datatype="A" format="YYYY-MM-DD hh:mm:ss" name="DATE_UTC" width="19">
        <DESCRIPTION>UTC Date</DESCRIPTION>
      </FIELD>
      <FIELD datatype="A" format="RAh:RAm:RAs" name="RA_J2000" unit="h" width="11">
        <DESCRIPTION>Right ascension of target</DESCRIPTION>
      </FIELD>
      <FIELD datatype="A" format="DEd:DEm:DEs" name="DEC_J2000" unit="deg" width="11">
        <DESCRIPTION>Declination of target</DESCRIPTION>
      </FIELD>
      <!--Data table-->
      <DATA>
        <CSV colsep="|" headlines="4">
<![CDATA[
DATE_UTC           |RA_J2000   |DEC_J2000  |
YYYY-MM-DD hh:mm:ss|RAh:RAm:RAs|DEd:DEm:DEs|
0123456789012345678|01234567890|01234567890|
-------------------|-----------|-----------|
2014-12-28 09:59:59|01:27:30.26|+13:40:30.6|
]]>        </CSV>
      </DATA>
    </TABLE>
  </ASTRO>
"""

class TestEphemTarget(TestCase):

    def test_create(self):
        """Make sure we can build an object with just a name."""
        target_name = "test"
        et = ephem_target.EphemTarget(target_name)
        self.assertIn(target_name, et.name)

    def test_precision_and_wrap(self):
        """Check that the precision in the output fits within allowed space and values to round to 60s"""
        target_name = "test"
        coordinate = coordinates.SkyCoord("11:59:59.9999999999 -10:00:59.9999999999",
                                          unit=(units.hour, units.degree),
                                          obstime=Time("2000-01-01 10:00:00", scale='utc'))
        et = ephem_target.EphemTarget(target_name)
        et.append(coordinate)
        line = et.doc.getElementsByTagName("CSV")[0].firstChild.wholeText.split('\n')[-2]
        values = line.split('|')
        self.assertEqual(len(values), 4)
        self.assertEqual(values[1], "12:00:00.00")
        self.assertEqual(values[2], "-10:01:00.0")

    def test_file_write(self):
        """Check that we are writing the file."""

        et = ephem_target.EphemTarget("Test")
        coordinate = coordinates.SkyCoord("01:27:30.26 +13:40:30.60",
                                          unit=(units.hour, units.degree),
                                          obstime=Time("2014-12-28 09:59:59", scale='utc'))
        et.append(coordinate)
        # coordinate = coordinates.SkyCoord("01:27:29.39 +13:40:20.90",
        #                                   unit=(units.hour, units.degree),
        #                                   obstime=Time("2014-12-29 09:59:59", scale='utc'))
        # et.append(coordinate)
        print et.coordinates
        f_handle = cStringIO.StringIO()
        et.writer(f_handle)
        f_handle.seek(0)
        self.assertEqual(f_handle.read(), example_file)
        f_handle.close()



from math import sqrt
import operator
import math
import sys
import os
import string

from matplotlib.patches import Rectangle
from matplotlib.patches import Ellipse
from matplotlib import rcParams
import numpy as np
from matplotlib.pyplot import figure, savefig
from astropy.io import votable
import ephem
import Polygon
import Polygon.IO

import invariable
import mpcread
import usnoB1
import megacam
from ossos import storage
from ossos import orbfit
from ossos import mpc


USER = os.getegid('HOME','/Users/michele/')

# USER = '/Users/jjk/'

def plot_line(axes, fname, ltype):
    """plot the ecliptic plane line on the given axes."""
    x = np.genfromtxt(fname, unpack=True)
    axes.plot(x[0], x[1], ltype)


MPCORB_FILE = os.path.join(USER, 'MPCORB-Distant.dat')
# MPCORB_FILE = os.path.join(USER, 'Dropbox/Develop/code/MPCORB.DAT')
L7MODEL = 'vos:OSSOS/CFEPS/L7SyntheticModel-v09.txt'
# L7MODEL = '/Users/kavelaarsj/Dropbox/Research/KuiperBelt/OSSOS/L7SyntheticModel-v09.txt'
# REAL_KBO_AST_DIR = '/Users/jjk/Dropbox/dbaseclone/dbaseclone/ast/'
REAL_KBO_AST_DIR = os.path.join(USER, 'Dropbox/OSSOS/measure3/ossin/')

PLOT_FIELD_EPOCH = 'Nov15'  # Oct14.00 ==> '0' days since the New Moon on Oct14
#TODO the .00 is appended when this variable is used as a keyword that needs that .00 this is bad.
DISCOVERY_NEW_MOON = 'Nov15'  # this is the date that the RA/DEC in blocks corresponds to.

PLOT_USNO_STARS = True
PLOT_MEGACAM_ARCHIVE_FIELDS = True
PLOT_SYNTHETIC_KBOS = True
PLOT_SYNTHETIC_KBO_TRAILS = False
PLOT_REAL_KBOS = True and os.access(REAL_KBO_AST_DIR, os.F_OK)
PLOT_FIELD_LAYOUT = True

PLOT_MPCORB = False and os.access(MPCORB_FILE, os.F_OK)


LABEL_FIELDS = True
LABEL_PLANETS = True
# # EmulateApJ columnwidth=245.26 pts
fig_width_pt = 246.0 * 3.0
inches_per_pt = 1.0 / 72.27
golden_mean = (sqrt(5.) - 1.0) / 2.0
fig_width = fig_width_pt * inches_per_pt
fig_height = fig_width * golden_mean * 1.5
fig_size = [fig_width, fig_height]

params = {'backend': 'pdf',
          'axes.labelsize': 12,
          'text.fontsize': 12,
          'legend.fontsize': 10,
          'xtick.labelsize': 10,
          'ytick.labelsize': 10,
          'text.usetex': False,
          'font.serif': 'Times',
          'font.family': 'serif',
          'image.aspect': 'equal',
          'figure.subplot.left': 0.2,
          'figure.subplot.bottom': 0.15,
          'figure.figsize': fig_size}

rcParams.update(params)

etHeader = """<?xml version = "1.0"?>
<!DOCTYPE ASTRO SYSTEM "http://vizier.u-strasbg.fr/xml/astrores.dtd">
<ASTRO ID="v0.8" xmlns:ASTRO="http://vizier.u-strasbg.fr/doc/astrores.htx">
  <TABLE ID="Table">
    <NAME>Ephemeris</NAME>
    <TITLE>Ephemeris for CFHT QSO</TITLE>
    <!-- Definition of each field -->
    <FIELD name="DATE_UTC"  datatype="A" width="19" format="YYYY-MM-DD hh:mm:ss"> 
        <DESCRIPTION>UTC Date</DESCRIPTION>
    </FIELD>
    <FIELD name="RA_J2000"  datatype="A" width="11" unit="h"   format="RAh:RAm:RAs">
        <DESCRIPTION>Right ascension of target</DESCRIPTION>
    </FIELD>
    <FIELD name="DEC_J2000" datatype="A" width="11" unit="deg" format="DEd:DEm:DEs">
        <DESCRIPTION>Declination of target</DESCRIPTION>
    </FIELD>
    <!-- Data table -->
<DATA><CSV headlines="4" colsep="|">
<![CDATA[
DATE_UTC           |RA_J2000   |DEC_J2000  |
YYYY-MM-DD hh:mm:ss|hh:mm:ss.ss|+dd:mm:ss.s|
1234567890123456789|12345678901|12345678901|
-------------------|-----------|-----------|
"""

header = """<?xml version = "1.0"?>
<!DOCTYPE ASTRO SYSTEM "http://vizier.u-strasbg.fr/xml/astrores.dtd">
<ASTRO ID="v0.8" xmlns:ASTRO="http://vizier.u-strasbg.fr/doc/astrores.htx">
<TABLE ID="Table">
<NAME>Fixed Targets</NAME>
<TITLE>Fixed Targets for CFHT QSO</TITLE>
<!-- Definition of each field -->
<FIELD name="NAME" datatype="A" width="20">
   <DESCRIPTION>Name of target</DESCRIPTION>
</FIELD>
<FIELD name="RA" ref="" datatype="A" width="11" unit="&quot;h:m:s&quot;">
   <DESCRIPTION>Right ascension of target</DESCRIPTION>
</FIELD>     
<FIELD name="DEC" ref="" datatype="A" width="11" unit="&quot;d:m:s&quot;">
   <DESCRIPTION>Declination of target</DESCRIPTION>
</FIELD>     
<FIELD name="EPOCH" datatype="F" width="6">
    <DESCRIPTION>Epoch of coordinates</DESCRIPTION>
</FIELD>     
<FIELD name="POINT" datatype="A" width="5">
<DESCRIPTION>Pointing name</DESCRIPTION>
</FIELD>     
<!-- Data table --> 
<DATA><CSV headlines="4" colsep="|"><![CDATA[
NAME                |RA         |DEC        |EPOCH |POINT|
                    |hh:mm:ss.ss|+dd:mm:ss.s|      |     |
12345678901234567890|12345678901|12345678901|123456|12345|
--------------------|-----------|-----------|------|-----|
"""

# spring13
# blocks = {#'13AE': {"RA": "14:15:28.89", "DEC": "-12:32:28.4"},  # E+0+0: image 1616681, ccd21 on April 9
#'13AO': {"RA": "15:58:01.35", "DEC": "-12:19:54.2"},  # O+0+0: image 1625346, ccd21 on May 8
#'14AN': {'RA': "15:30:00.00", "DEC": "-11:00:00.0"},
#'14AM': {'RA': "15:30:00.00", "DEC": "-12:20:00.0"},
#'14AS': {'RA': "15:12:00.00", "DEC": "-17:40:00.0"}}


#fall13
# blocks = {'13BL': {'RA': "00:54:00.00", "DEC": "+03:50:00.00"}}  # ,
# '13AE': {"RA": "14:15:28.89", "DEC": "-12:32:28.4"},  # E+0+0: image 1616681, ccd21 on April 9
#           '13AO': {"RA": "15:58:01.35", "DEC": "-12:19:54.2"},  # O+0+0: image 1625346, ccd21 on May 8
#           '13BH': {'RA': "01:30:00.00", "DEC": "+13:00:00.00"},
#           '14AM': {'RA': "15:30:00.00", "DEC": "-12:20:00.0"}
#           }

## this position is derived from the +0+0 field for 13B observed on November 1st 2013
# blocks = {'14BH': {'RA': "01:28:32.32", "DEC": "+12:51:06.10"}}

#fall 2015
blocks = {'15BD': {'RA': "03:15:00.00", "DEC": "+16:30:00.00"}}

#spring15
# blocks = {'15AP': {'RA': "13:30:00", "DEC": "-07:45:00"}}
# blocks = {'14AM': {'RA': "15:30:00.00", "DEC": "-12:20:00.0"}}  # the centre when set in May 2014.
# will then define a 15AM to work properly.
# blocks = {'15AM': {'RA': "15:35:00.00", "DEC": "-12:10:00.0"}}  # the centre for discovery in May 2015.
# blocks = {'15AM': {'RA': "15:35:00.00", "DEC": "-12:10:00.0"}}  # the centre for discovery in May 2015.
# blocks = {
# '14BH': {'RA': "01:28:32.32", "DEC": "+12:51:06.10"},
#     '13BL': {'RA': "00:54:00.00", "DEC": "+03:50:00.00"},
#     '15BS': {'RA': "00:30:00.00", "DEC": "+04:45:00.00"}}

blocks = {
    # E+0+0: image 1616681, ccd21 on April 9. E block discovery triplets are April 4,9,few 19
    # '13AE': {"RA": "14:15:28.89", "DEC": "-12:32:28.4"},
    #    '13AO': {"RA": "15:58:01.35", "DEC": "-12:19:54.2"},  # O+0+0: image 1625346, ccd21 on May 8. O block are
    # May 7,8.
    # '13BL': {'RA': "00:54:00.00", "DEC": "+03:50:00.00"},  # 13B blocks are at their opposition locations
    # '14BH': {'RA': "01:30:00.00", "DEC": "+13:00:00.00"},  # due to bad weather, discovery wasn't until 2014, so 14

    #    '15AP': {'RA': "13:30:00.00", "DEC": "-7:45:00.00"},  # on-plane
    # '15AM': {'RA': "15:35:00.00", "DEC": "-12:10:00.0"}  # positioned for its 2015 discovery opposition.
    '15BS': {'RA': "00:30:00.00", "DEC": "+05:00:00.00"},  # rejected: dec "-02:45:00.00"
    #     '15BD': {'RA': "03:15:00.00", "DEC": "+16:30:00.00"}
}

newMoons = {
#    'Feb13': "2013/02/10 10:00:00",
#    'Mar13': "2013/03/11 10:00:00",
#    'Apr13': "2013/04/10 10:00:00",
#    'May13': "2013/05/09 10:00:00",
#    'Jun13': "2013/06/08 10:00:00",
#    'Jul13': "2013/07/08 10:00:00",
#    'Aug13': "2013/08/06 10:00:00",
#    'Sep13': '2013/09/05 10:00:00',
    # 'Oct13': '2013/10/04 10:00:00',
#    'Nov13': '2013/11/03 10:00:00',
#    'Dec13': '2013/12/02 10:00:00',
#    'Jan14': '2014/01/01 10:00:00',
#    'Feb14': '2014/01/31 10:00:00',
#    'Mar14': '2014/03/28 10:00:00',
#    'Apr14': '2014/04/01 10:00:00',
# 'May14': '2014/05/28 10:00:00',
#    'Jun14': '2014/06/26 10:00:00',
#    'Jul14': '2014/07/26 10:00:00',
#    'Aug14': "2014/08/25 10:00:00",
#    'Sep14': '2014/09/24 10:00:00',
#    'Oct14': '2014/10/23 10:00:00',
#    'Nov14': '2014/11/22 10:00:00',
#    'Dec14': '2014/12/22 10:00:00',
#    'Jan15': '2015/01/20 10:00:00',
#    'Feb15': '2015/02/18 10:00:00',
#    'Mar15': '2015/03/19 10:00:00',
#    'Apr15': '2015/04/18 10:00:00',
#    'May15': '2015/05/17 10:00:00',
#    'Jun15': '2015/06/16 10:00:00',
    'Jul15': '2015/07/15 10:00:00',
    'Aug15': '2015/08/14 10:00:00',
    'Sep15': '2015/09/12 10:00:00',
    'Oct15': '2015/10/12 10:00:00',
    'Nov15': '2015/11/11 10:00:00',
    'Dec15': '2015/12/11 10:00:00',
    'Jan16': '2016/01/09 10:00:00',
    'Feb16': '2016/03/08 10:00:00',
    'Mar16': '2016/03/09 10:00:00',
    'Apr16': '2016/04/08 10:00:00',
    'May16': '2016/05/07 10:00:00',
    'Jun16': '2016/06/05 10:00:00',
    'Jul16': '2016/07/05 10:00:00',
    'Aug16': '2016/08/03 10:00:00',
    'Sep16': '2016/09/01 10:00:00',
    'Oct16': '2016/10/01 10:00:00',
    'Nov16': '2016/11/01 10:00:00',
    'Dec16': '2016/12/01 10:00:00',
    'Jan17': '2017/01/01 10:00:00',
    # 'Feb17': '2017/02/01 10:00:00',
    }



xgrid = {'2014': [-3, -2, -1, 0, 1, 2, 3],
         '2014r': [-3.5, -2.5, -1.5, -0.5, 0.5, 1.5, 2.5, 3.5],
         'astr': [1, 2, 3, 4],
         '40ccd': [2, 1, 0, -1, -2]}
# 2013
#       'astr':[-5.5,-4.5,-3.5,-2.5,-1.5,-0.5,0.5,1.5,2.5,3.5,4.5,5.5]}

ygrid = {'2014': [-1, 0, 1],
         'astr': [-2.5, -1.5, -0.5, 0.5, 1.5],
         '2014r': [-1.5, -0.5, 0.5, 1.5],
         '40ccd': [-1.5, -0.5, 0.5, 1.5]}
# 2013
#       'astr':  [-2.2,-1.2, -0.2, 0.8, 1.8],

block_centre = ephem.Ecliptic(0, 0)
field_centre = ephem.Ecliptic(0, 0)

## foffset is the offset (in RA and DEC) between fields
field_offset = math.radians(1.00)
# which year, for the x/y offset grids, are we using.
year = '40ccd'

# the 'astr' year indicates astrometric grid,  this should have more overlap so reduce the field_offset for those.
if year == 'astr':
    field_offset *= 0.75

## camera_dimen is the size of the field.
## this is the 40 CD Layout.  The width is set to exlcude one of the 
## Ears.  dimen is really the height
pixscale = 0.184
detsec_ccd09 = ((4161, 2114), (14318, 9707))
detsec_ccd10 = ((6279, 4232), (14317, 9706))
detsec_ccd36 = ((2048, 1), (14318, 9707))
detsec_ccd37 = ((23219, 21172), (14309, 9698))
detsec_ccd00 = ((4160, 2113), (19351, 14740))
detsec_ccd17 = ((21097, 19050), (14309, 9698))
detsec_ccd35 = ((16934, 18981), (11, 4622))

print detsec_ccd09[0][0] - detsec_ccd10[0][1], detsec_ccd09[0][1] - detsec_ccd36[0][0]

camera_width =  (max(detsec_ccd37[0]) - min(detsec_ccd09[0]) + 1 )* pixscale / 3600.0
camera_height = (max(detsec_ccd00[1]) - min(detsec_ccd35[1]) + 1 )* pixscale / 3600.0
camera_width_40 =  (max(detsec_ccd37[0]) - min(detsec_ccd36[0]) + 1 )* pixscale / 3600.0
camera_width_36 = (max(detsec_ccd17[0]) - min(detsec_ccd09[0]) + 1 ) * pixscale / 3600.0 

print camera_width, camera_width_40, camera_width_36, camera_height

years = {"2014": {"ra_off": ephem.hours("00:00:00"),
                  "dec_off": ephem.hours("00:00:00"),
                  "fill": False,
                  "facecolor": 'k',
                  "alpha": 0.5,
                  "color": 'b'},
         "40ccd": {"ra_off": ephem.hours("00:00:00"),
                  "dec_off": ephem.hours("00:00:00"),
                  "fill": False,
                  "facecolor": 'k',
                  "alpha": 0.5,
                  "color": 'b'},
         "2014r": {"ra_off": ephem.hours("00:05:00"),
                   "dec_off": ephem.degrees("00:18:00"),
                   "alpha": 0.5,
                   "fill": False,
                   "facecolor": 'k',
                   "color": 'g'},
         'astr': {"ra_off": ephem.hours("00:00:00"),
                  "dec_off": ephem.degrees("-00:12:00"),
                  "alpha": 0.05,
                  "fill": True,
                  'facecolor': 'k',
                  "color": 'none'},
         }

# coverage is going to contain the polygons that describe our individual fields.
coverage = []

# ras/decs will hold the centers of the search fields, in degrees.
ras = []
decs = []

fix = open('Fall14.xml', 'w')
fix.write(header)

# build the pointings that will be used for the discovery fields
# This is a bit tricky as we keep the heliocentric longitude constant while moving in RA
# and get latitude coverage by changing the DEC.
# Compute the coverage of the discovery fields.
field_names = []
field_polygon = None

for block in blocks.keys():
    ra_start = ephem.hours(blocks[block]["RA"]) + years[year]["ra_off"]
    dec_start = ephem.degrees(blocks[block]["DEC"]) + years[year]["dec_off"]
    height = math.radians(camera_height)
    idy = 0
    for dx in xgrid[year]:
        idx = int(dx)
        for dy in ygrid[year]:
            idy = int(math.floor(dy))
            decc = ephem.degrees(dec_start + dx*height/2.0)
            this_decc = decc + dy * height
            width = 1.007*math.radians(camera_width / math.cos(this_decc))
            ## set to a -ve instead of +ve for the 'fall'
            rac = ephem.hours(ra_start - dx * width)
            dec = math.degrees(this_decc)
            ra = math.degrees(rac)
            print "{} {} {}".format(rac, dec, math.degrees(width))
            field_name = "%s%+d%+d" % (block, idx, idy)
            field_names.append(field_name)
            decs.append(np.radians(dec))
            ras.append(np.radians(ra))
            xcen = ra
            ycen = dec
            fix.write('%20s|%10s|%10s|2000.0|    1|\n' % (field_name,
                                                          ephem.hours(math.radians(ra)),
                                                          ephem.degrees(math.radians(dec))))
            dimenw = math.degrees(width)
            dimenh = math.degrees(height)
            this_point_polygon = Polygon.Polygon((
                (xcen - dimenw / 2.0, ycen - dimenh / 2.0),
                (xcen - dimenw / 2.0, ycen + dimenh / 2.0),
                (xcen + dimenw / 2.0, ycen + dimenh / 2.0),
                (xcen + dimenw / 2.0, ycen - dimenh / 2.0),
                (xcen - dimenw / 2.0, ycen - dimenh / 2.0)))
            field_polygon = field_polygon is None and this_point_polygon or this_point_polygon | field_polygon


fix.write("""]]</CSV></DATA>
</TABLE>
</ASTRO>
""")
fix.close()

field_polygon.simplify()
print "Field Area: {} degrees squared".format(field_polygon.area())
print "Field Centre: {} ".format(field_polygon.center())

(ra_min, ra_max, dec_min, dec_max) = field_polygon.boundingBox()

if year == "astr":
    ## we don't do trailing for the astr ometric grid, or make any plots...
    sys.exit(0)

ras = np.array(ras)
decs = np.array(decs)
ra_cen = math.degrees(ras.mean())
dec_cen = math.degrees(decs.mean())
#ra_cen = 15.0
#dec_cen = 5.0
#width = 245 - 205
#height = -8 + 25
#ra_cen = 180.0
#dec_cen = 0.0
#width = 360
#height = 70
#ra_cen = 45.0
#dec_cen = 17.5

### These values are the half width/height of the field.
width = 24
height = 12

## lets make a plot of the field selections.
fig = figure()
ax = fig.add_subplot(111)
# ax.set_aspect('equal')

# xylims = [250, 195, -17, -4]  # all of A semester
# xylims = [27, 3, -6, 17]  # H, L and low S blocks
xylims = [26, 3, 0, 16]  # H, L and high S blocks
# xylims = [54, 36, 12, 21]  # D block
ax.set_xlim(xylims[0], xylims[1])  # ra_cen + width, ra_cen - width)
ax.set_ylim(xylims[2], xylims[3])  # dec_cen - height, dec_cen + height)
#ax.set_ylim(-30,30)
ax.set_xlabel('RA (deg)')
ax.set_ylabel('DE (deg)')

ax.grid()


## plot the galactic plane line ..
plot_line(ax, 'eplane.radec', 'b-')
plot_line(ax, 'gplane.radec', 'g-')
ax.text(20, 8.5, 'ecliptic', fontdict={'color': 'b'})

# plot the invariant plane
lon = np.arange(-2 * math.pi, 2 * math.pi, 0.5 / 57)
lat = 0 * lon
(lat, lon) = invariable.trevonc(lat, lon)

ec = [ephem.Ecliptic(x, y) for (x, y) in np.array((lon, lat)).transpose()]
eq = [ephem.Equatorial(coord) for coord in ec]
ax.plot([math.degrees(coord.ra) for coord in eq],
        [math.degrees(coord.dec) for coord in eq],
        '-k',
        lw=1,
        alpha=0.7)
ax.text(25, 7.5, 'invariant', fontdict={'color': 'k'})

(cc_lat, cc_lon) = invariable.trevonc(lat, lon, chiangchoi_plane=True)
ec = [ephem.Ecliptic(x, y) for (x, y) in np.array((cc_lon, cc_lat)).transpose()]
eq = [ephem.Equatorial(coord) for coord in ec]
ax.plot([math.degrees(coord.ra) for coord in eq],
        [math.degrees(coord.dec) for coord in eq],
        '.r',
        lw=1,
        alpha=0.7)
ax.text(25, 6, 'Chiang_Choi', fontdict={'color': 'r'})


## build a list of Synthetic KBOs that will be in the discovery fields.
print "LOADING SYNTHETIC MODEL KBOS FROM: {}".format(L7MODEL)
ra = []
dec = []
kbos = []
lines = storage.open_vos_or_local(L7MODEL).read().split('\n')

# Look for synthetic KBOs that are in the field on this date.
discovery_date = ephem.date(newMoons[DISCOVERY_NEW_MOON])
plot_date = ephem.date(newMoons[PLOT_FIELD_EPOCH])

for line in lines:
    if len(line) == 0 or line[0] == '#':  # skip initial column descriptors and the final blank line
        continue
    kbo = ephem.EllipticalBody()
    values = line.split()
    kbo._a = float(values[0])
    kbo._e = float(values[1])
    kbo._inc = float(values[2])
    kbo._Om = float(values[3])
    kbo._om = float(values[4])
    kbo._M = float(values[5])
    kbo._H = float(values[6])
    epoch = ephem.date(2453157.50000 - ephem.julian_date(0))
    kbo._epoch_M = epoch
    kbo._epoch = epoch
    kbo.name = values[8]
    kbo.compute(discovery_date)

    kbo_ra = math.degrees(kbo.ra)
    kbo_dec = math.degrees(kbo.dec)
    ## keep a list of KBOs that are in the discovery pointings
    if field_polygon.isInside(math.degrees(float(kbo.ra)), math.degrees(float(kbo.dec))):
        ### only keep objects that are brighter than limit
        if kbo.mag < 25.0:
            kbos.append(kbo)
            if PLOT_SYNTHETIC_KBOS:
                kbo.compute(plot_date)
                ax.scatter(math.degrees(float(kbo.ra)), math.degrees(float(kbo.dec)), c='k', marker='o', s=2, alpha=0.8)

print "{} KBOs found in coverage on {}".format(len(kbos), discovery_date)
## Now we work out how far to move the fields at different lunations
seps = {}
dates = {}

## build a list of dates that we will need field locations for.
for month in newMoons:
    for night in range(-8, 9):
        epoch = "%s.%s" % (month, string.zfill(night, 2))
        dates[epoch] = ephem.date(ephem.date(newMoons[month]) + night)
        seps[epoch] = {'dra': 0, 'ddec': 0}

print "COMPUTING NOMINAL MOTIONS USING SYNTHETIC MODEL KBOS FROM: {}".format(L7MODEL)

## Compute the typical motion of KBOs that were in the discovery field.
for kbo in kbos:
    kbo.compute(discovery_date)
    ra = kbo.ra
    dec = kbo.dec
    for epoch in dates:
        date = dates[epoch]
        kbo.compute(date)
        seps[epoch]['dra'] += kbo.ra - ra
        seps[epoch]['ddec'] += kbo.dec - dec

## plot source locations at the start
## middle and end of semester
if PLOT_SYNTHETIC_KBO_TRAILS:
    print "PLOTTING TRAILS USING SYNTHETIC MODEL KBOS FROM: {}".format(L7MODEL)
    colours = {'Sep14': 'g', 'Oct14': 'b', 'Nov14': 'r'}
    alpha = {'Sep14': 0.3, 'Oct14': 0.7, 'Nov14': 0.3}
    zorder = {'Sep14': 1, 'Oct14': 5, 'Nov14': 2}
    for month in ['Sep14', 'Oct14', 'Nov14']:
        ra = []
        dec = []
        date = ephem.date(newMoons[month])
        for kbo in kbos:
            kbo.compute(date)
            ra.append(math.degrees(kbo.ra))
            dec.append(math.degrees(kbo.dec))
            ax.plot(ra, dec, colours[month] + '.', alpha=alpha[month], zorder=zorder[month])

# compute the mean of the separation between the location at discovery and the location at epoch.
for month in seps:
    seps[month]['dra'] /= float(len(kbos))
    seps[month]['ddec'] /= float(len(kbos))


sorted_epochs = sorted(dates.iteritems(), key=operator.itemgetter(1))

camera_width=0.983*camera_width
camera_height=0.983*camera_height

print "CREATING ET XML FILES USING BASE POINTING AND NOMINAL MOTION RATES"
for idx in range(len(ras)):
    name = field_names[idx]
    f = file('%s.xml' % name, 'w')
    f.write(etHeader)
    for epoch in sorted_epochs:
        epoch = epoch[0]
        date = dates[epoch]
        ra = ras[idx] + seps[epoch]['dra']
        dec = decs[idx] + seps[epoch]['ddec']
        tdate = date.tuple()
        zf = string.zfill
        sdate = "%4s-%2s-%2s %2s:%2s:%2s" % (
            tdate[0], zf(tdate[1], 2), zf(tdate[2], 2), zf(tdate[3], 2), zf(tdate[4], 2), zf(int(tdate[5]), 2))
        f.write('%19s|%11s|%11s|\n' % (sdate, ephem.hours(ra), ephem.degrees(dec)))
        if epoch == PLOT_FIELD_EPOCH+".00" and PLOT_FIELD_LAYOUT:
            ax.add_artist(Rectangle(xy=(math.degrees(ra) - camera_width_40/2.0, math.degrees(dec) - camera_height / 4.0 ),
                                    width= camera_width_40,
                                    height= camera_height/2.0,
                                    color='b',
                                    lw=0.5, fill=False, alpha=0.3)) 
            ax.add_artist(Rectangle(xy=(math.degrees(ra) - camera_width_36/2.0, math.degrees(dec) - camera_height / 2.0),
                                    height=camera_height/4.0,
                                    width=camera_width_36,
                                    color='g',
                                    lw=0.5, fill=False, alpha=0.3))
            ax.add_artist(Rectangle(xy=(math.degrees(ra) - camera_width_36/2.0, math.degrees(dec) + camera_height / 4.0),
                                    height=camera_height/4.0,
                                    width=camera_width_36,
                                    color='r',
                                    lw=0.5, fill=False, alpha=0.3))
            if LABEL_FIELDS:
                ax.text(math.degrees(ra), math.degrees(dec),
                        name,
                        horizontalalignment='center',
                        verticalalignment='center',
                        zorder=10,
                        fontdict={'size': 4, 'color': 'darkblue'})


    f.write("""]]</CSV></DATA>\n</TABLE>\n</ASTRO>\n""")
    f.close()


if PLOT_USNO_STARS:
    print "PLOTTING LOCATIONS NEARBY BRIGHT USNO B1 STARS"
    for ra in range(int(ra_cen - width), int(ra_cen + width), 10):
        for dec in range(int(dec_cen - height), int(dec_cen + height), 10):
	    file_name = USER + "new_usno/usno{:5.2f}{:5.2f}.xml".format(ra, dec).replace(" ", "")
            print file_name
            file_name = file_name.replace(" ","")
	    if not os.access(file_name, os.R_OK):
                usno = usnoB1.TAPQuery(ra, dec, 10.0, 10.0)
		fobj=open(file_name,'w')
	        fobj.write(usno.read())
                fobj.close()
            try:
               t = votable.parse(open(file_name,'r')).get_first_table()
            except:
               print "No USNO stars found for: {} {}\n".format(ra,dec)
               continue
	    select = t.array['Bmag'] < 9.3
            Rmag = t.array['Bmag'][select]
            min_mag = max(Rmag.min(), 11)
            scale = 0.5 * 10 ** ((min_mag - Rmag) / 2.5) 
            ax.scatter(t.array['RAJ2000'][select], t.array['DEJ2000'][select], s=scale, marker='o', facecolor='y', alpha=0.3, edgecolor='',
                       zorder=-10)

    for planet in [ephem.Mars(), ephem.Jupiter(), ephem.Saturn(), ephem.Uranus(), ephem.Neptune()]:
        planet.compute(plot_date)
        ax.scatter(math.degrees(planet.ra), math.degrees(planet.dec),
                   marker='o',
                   s=40,
                   facecolor='r',
                   edgecolor='g', )
        if LABEL_PLANETS :
            ax.text(math.degrees(planet.ra), math.degrees(planet.dec),
                planet.name,
                horizontalalignment='center',
                fontdict={'size': 6,
                          'color': 'darkred'})

if PLOT_MEGACAM_ARCHIVE_FIELDS:
    print "PLOTTING FOOTPRINTS NEARBY ARCHIVAL MEGAPRIME IMAGES."

    for qra in xrange(int(xylims[1]), int(xylims[0]), 5):
        print qra
        for qdec in xrange(int(xylims[2]), int(xylims[3]), 10):
            print qra, qdec
            filename = USER + "new_usno/megacam{:+6.2f}{:+6.2f}.xml".format(qra, qdec)
            if not os.access(filename, os.R_OK):
                # TAP query will max out silently if return table too large. Make small units.
                data = megacam.TAPQuery(qra, qdec, 5.0, 10.0).read()
                # FIXME: add a catch for 503's when TAP fails when CADC drops connection
                fobj = open(filename, 'w')
                fobj.write(data)
                fobj.close()
            fobj = open(filename, 'r')
            t = votable.parse(fobj).get_first_table()

            ra = t.array['RAJ2000']
            dec = t.array['DEJ2000']

            rects = [Rectangle(xy=(ra[idx] - camera_width_36 / 2.0, dec[idx] - camera_width_36 / 2.0),
                               height=camera_height,
                               width=camera_width_36,
                               edgecolor='m',
                               alpha=0.1,
                               lw=0.1, zorder=-100,
                               fill=False) for idx in xrange(ra.size)]
            for r in rects:
                ax.add_artist(r)
                

if PLOT_MPCORB:
    print "PLOTTING LOCATIONS OF KNOWN KBOs (using {})".format(MPCORB_FILE)
    kbos = mpcread.getKBOs(MPCORB_FILE)
    for kbo in kbos:
        kbo.compute(plot_date)
        if field_polygon.isInside(math.degrees(float(kbo.ra)), math.degrees(float(kbo.dec))):
            ax.scatter(math.degrees(kbo.ra),
                       math.degrees(kbo.dec),
                       marker='x',
                       s=4,
                       facecolor='r',
                       edgecolor='r', alpha=0.3)

if PLOT_REAL_KBOS:
    reader = mpc.MPCReader()
    print "PLOTTING LOCATIONS OF OSSOS KBOs (using directory {})".format(REAL_KBO_AST_DIR)
    for ast in os.listdir(REAL_KBO_AST_DIR):
        if not ast.startswith('u'):
            obs = reader.read(REAL_KBO_AST_DIR + ast)
            try:
                kbo = orbfit.Orbfit(obs)
            except:
                continue
            kbo.predict(ephem.julian_date(ephem.date(plot_date)))
            # if not field_polygon.isInside(float(kbo.coordinate.ra.degrees), float(kbo.coordinate.dec.degrees)):
            #    continue
            if obs[0].mag < 23.6:
                c = 'b'
                if LABEL_FIELDS:
                    ax.text(kbo.coordinate.ra.degree,
                            kbo.coordinate.dec.degree,
                            ast.split('.')[0],
                            horizontalalignment='center',
                            verticalalignment='baseline',
                            fontdict={'size': 4,
                                      'color': 'darkred'})
            else:
                c = 'g'
                ax.text(kbo.coordinate.ra.degree,
                        kbo.coordinate.dec.degree,
                        ast.split('.')[0],
                        horizontalalignment='center',
                        verticalalignment='baseline',
                        fontdict={'size': 4,
                                  'color': 'darkred'})
            # from astropy import units
            ax.add_artist(Ellipse((kbo.coordinate.ra.degree, kbo.coordinate.dec.degree,),
                                  width=2.0 * kbo.dra / 3600.0,
                                  height=2.0 * kbo.ddec / 3600.0,
                                  angle=360 - (kbo.pa + 90),  # kbo.pa.to(units.degree).value
                                  edgecolor='none',
                                  facecolor='k',
                                  alpha=0.1))
            ax.scatter(kbo.coordinate.ra.degree,
                       kbo.coordinate.dec.degree,
                       marker='o',
                       s=1,
                       facecolor='none',
                       edgecolor=c,
                       alpha=0.5)

new_tick_locations = np.array(range(360,0,-30))
def tick_function(X):
    import calendar
    month = ( X/30.0 + 8 ) % 12 + 1
    return [ calendar.month_abbr[int(z)] for z in month ]
print new_tick_locations
if False:
    ax.set_xlim(360,0)
    ax2 = ax.twiny()
    ax2.set_xticks(new_tick_locations)
    ax2.set_xticklabels(tick_function(new_tick_locations))
    ax2.set_xlabel("Opp. Month")    
    ax2.set_xlim(360,0)


print "SAVING FILE"
savefig('layout40-at' + PLOT_FIELD_EPOCH + '-discov_on-' + DISCOVERY_NEW_MOON + '.pdf')

sys.stderr.write("FINISHED\n")



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, contains_inanyorder, equal_to

from tests.base_tests import FileReadingTestCase
from ossos.gui.context import LocalDirectoryWorkingContext


class LocalDirectoryContextTest(FileReadingTestCase):
    def test_directory_manager_get_listing(self):
        directory = self.get_abs_path("data/testdir")

        directory_manager = LocalDirectoryWorkingContext(directory)

        listing1 = directory_manager.get_listing("cands.astrom")
        assert_that(listing1, contains_inanyorder("xxx1.cands.astrom", "xxx2.cands.astrom"))

        listing2 = directory_manager.get_listing("reals.astrom")
        assert_that(listing2, contains_inanyorder("xxx1.reals.astrom", "xxx2.reals.astrom"))

        assert_that(directory_manager.get_full_path("xxx1.cands.astrom"),
                    equal_to(self.get_abs_path("data/testdir/xxx1.cands.astrom")))


if __name__ == '__main__':
    unittest.main()



#!python
import argparse
from astropy.io import votable
import warnings
from astropy.table import Table
import requests
from cStringIO import StringIO
from astropy.time import Time


def cfht_megacam_tap_query(ra_deg=180.0, dec_deg=0.0, width=1, height=1, date=None):
    """Do a query of the CADC Megacam table.

    Get all observations inside the box (right now it turns width/height into a radius, should not do this).

    @rtype : Table
    @param ra_deg: center of search region, in degrees
    @param dec_deg: center of search region in degrees
    @param width: width of search region in degrees
    @param height: height of search region in degrees
    @param date: ISO format date string.  Query will be +/- 0.5 days from date given.
    """

    radius = min(90, max(width, height) / 2.0)

    query = ("SELECT "
             "COORD1(CENTROID(Plane.position_bounds)) AS RAJ2000,"
             "COORD2(CENTROID(Plane.position_bounds)) AS DEJ2000,"
             "target_name "
             "FROM "
             "caom2.Observation as o "
             "JOIN caom2.Plane as Plane on o.obsID=Plane.obsID "
             "WHERE o.collection = 'CFHT' "
             "AND o.instrument_name = 'MegaPrime' "
             "AND INTERSECTS( CIRCLE('ICRS', %f, %f, %f), Plane.position_bounds ) = 1")

    query = query % (ra_deg, dec_deg, radius)

    if date is not None:
        mjd = Time(date, scale='utc').mjd
        query += " AND Plane.time_bounds_cval1 <= {} AND {} <= Plane.time_bounds_cval2 ".format(mjd+0.5, mjd-0.5)

    data = {"QUERY": query,
            "REQUEST": "doQuery",
            "LANG": "ADQL",
            "FORMAT": "votable"}

    url = "http://www.cadc.hia.nrc.gc.ca/tap/sync"

    warnings.simplefilter('ignore')
    ff = StringIO(requests.get(url, params=data).content)
    ff.seek(0)
    table = votable.parse(ff).get_first_table().to_table()
    assert isinstance(table, Table)
    return table


def main(ra_deg, dec_deg, width, height):
    print cfht_megacam_tap_query(ra_deg, dec_deg, width, height)

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('ra', type=float,
                        help="RA (in degrees) of the centre of the search area.")
    parser.add_argument('dec', type=float,
                        help="DEC (in degrees) of the centre of the search area.")
    parser.add_argument('width', nargs='?', type=float,
                        help="width (in degrees) of the search area.", default=10.0/60.0)
    parser.add_argument('height', nargs='?', type=float,
                        help="height (in degrees) of the search area. (default==width)",
                        default=None)

    args = parser.parse_args()
    if args.height is None:
        args.height = args.width
    main(args.ra, args.dec, args.width, args.height)



from multiprocessing import Process, Queue

from ossos import astrom
from ossos import storage

__author__ = "David Rusk <drusk@uvic.ca>"


def _generate_provisional_name(q, astrom_header, fits_header):
    """
    Generates a name for an object given the information in its astrom
    observation header and FITS header.
    :param q: a queue of provisional names to return.
    :type q: Queue
    :param astrom_header:
    :param fits_header:
    """
    while True:
        ef = get_epoch_field(astrom_header, fits_header)
        epoch_field = ef[0] + ef[1]
        count = storage.increment_object_counter(storage.MEASURE3, epoch_field)
        try:
            q.put(ef[1] + count)
        except:
            break


def get_epoch_field(astrom_header, fits_header):
    # Format: "YYYY MM DD.dddddd"
    date = astrom_header[astrom.MJD_OBS_CENTER]
    year, month, _ = date.split()

    semester = "A" if 2 <= int(month) <= 7 else "B"

    epoch = year[-2:] + semester

    object_header = fits_header["OBJECT"]

    if object_header.startswith(epoch):
        field = object_header[len(epoch)]
    else:
        field = object_header[0]

    return epoch, field


class ProvisionalNameGenerator(object):
    """
    Creates provisional names for a new sources.

    These names must be stored in a 7-character packed form:
    http://www.minorplanetcenter.net/iau/info/PackedDes.html
    """

    def __init__(self):
        self._astrom_header = None
        self._fits_header = None
        self.provisional_name_queue = None
        self.p = None

    def generate_name(self, astrom_header, fits_header):
        if not self._astrom_header or self._astrom_header != astrom_header:
            self._astrom_header = astrom_header
            self._fits_header = fits_header
            if self.provisional_name_queue is not None:
                self.provisional_name_queue.close()
            if self.p is not None:
                self.p.terminate()
            self.provisional_name_queue = Queue(1)
            self.p = Process(target=_generate_provisional_name, args=(self.provisional_name_queue,
                                                                 self._astrom_header,
                                                                 self._fits_header))
            self.p.start()
        return self.provisional_name_queue.get()


class DryRunNameGenerator(ProvisionalNameGenerator):
    """
    Generate a fake name for dry runs so we don't increment counters.
    """
    def generate_name(self, astrom_header, fits_header):
        epoch_field = get_epoch_field(astrom_header, fits_header)
        count = storage.increment_object_counter(storage.MEASURE3,
                                                 epoch_field,
                                                 dry_run=True)

        base = "DRY"
        return base + count.zfill(7 - len(base))



__author__ = "David Rusk <drusk@uvic.ca>"

import os

from ossos import storage
from ossos.gui.progress import LocalProgressManager, VOSpaceProgressManager


def get_context(directory, userid=None):
    if directory.startswith("vos:"):
        return VOSpaceWorkingContext(directory, userid=userid)
    else:
        return LocalDirectoryWorkingContext(directory, userid=userid)


class WorkingContext(object):
    def __init__(self, directory, userid=None):
        self.userid = userid
        self.directory = directory

    def is_remote(self):
        raise NotImplementedError()

    def get_full_path(self, filename):
        return os.path.join(self.directory, filename)

    def listdir(self):
        raise NotImplementedError()

    def get_listing(self, suffix, exclude_prefix=None):
        if not exclude_prefix:
            name_match = lambda x: x.endswith(suffix)
        else:
            name_match = lambda x: x.endswith(suffix) and not x.startswith(exclude_prefix)
        return filter(name_match, self.listdir())

    def get_file_size(self, filename):
        raise NotImplementedError()

    def exists(self, filename):
        raise NotImplementedError()

    def open(self, filename):
        raise NotImplementedError()

    def rename(self, old_name, new_name):
        raise NotImplementedError()

    def remove(self, filename):
        raise NotImplementedError()

    def get_progress_manager(self):
        raise NotImplementedError()


class LocalDirectoryWorkingContext(WorkingContext):
    def __init__(self, directory, userid=None):
        super(LocalDirectoryWorkingContext, self).__init__(directory, userid=userid)

    def is_remote(self):
        return False

    def listdir(self):
        return os.listdir(self.directory)

    def get_file_size(self, filename):
        return os.stat(self.get_full_path(filename)).st_size

    def exists(self, filename):
        return os.path.exists(self.get_full_path(filename))

    def open(self, filename):
        filehandle = open(self.get_full_path(filename), 'a+b')

        # Note: Linux starts the file position at 0, but on MacOSX it
        # starts at the end of the file, so this makes things consistent
        # and allows us to subsequently call a read with the expected
        # results.
        filehandle.seek(0)

        return filehandle

    def rename(self, old_name, new_name):
        os.rename(self.get_full_path(old_name), self.get_full_path(new_name))

    def remove(self, filename):
        os.remove(self.get_full_path(filename))

    def get_progress_manager(self):
        return LocalProgressManager(self, userid=self.userid)


class VOSpaceWorkingContext(WorkingContext):
    def __init__(self, directory, userid=None):
        super(VOSpaceWorkingContext, self).__init__(directory, userid=userid)

    def is_remote(self):
        return True

    def listdir(self):
        # Don't force because it causes a HUGE performance hit.
        return storage.listdir(self.directory, force=False)

    def get_file_size(self, filename):
        length_property = storage.get_property(
            self.get_full_path(filename), "length", ossos_base=False)

        if length_property is None:
            # Fall-back if the length property is not set for some reason
            return self._get_file_size_by_downloading(filename)

        return int(length_property)

    def _get_file_size_by_downloading(self, filename):
        filehandle = storage.vofile(self.get_full_path(filename), os.O_RDONLY)
        contents = filehandle.read()
        filehandle.close()
        return len(contents)

    def exists(self, filename):
        return storage.exists(self.get_full_path(filename))

    def open(self, filename):
        return storage.vofile(self.get_full_path(filename), os.O_RDONLY)

    def rename(self, old_name, new_name):
        storage.move(old_name, new_name)

    def remove(self, filename):
        storage.delete_uri(self.get_full_path(filename))

    def get_progress_manager(self):
        return VOSpaceProgressManager(self, track_partial_progress=False, userid=self.userid)



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from mock import Mock
from hamcrest import assert_that, equal_to, not_none

from tests.base_tests import WxWidgetTestCase
from ossos.gui.views.navigation import NavPanel


class NavigationPanelTest(WxWidgetTestCase):
    def setUp(self):
        super(NavigationPanelTest, self).setUp()

        self.controller = Mock()
        self.undertest = NavPanel(self.rootframe, self.controller)

    def test_press_next_source_button(self):
        next_button = self.get_child_by_label(self.undertest, NavPanel.NEXT_LABEL)
        assert_that(next_button, not_none())

        self.fire_button_click_event(next_button)
        assert_that(self.controller.on_next_obs.call_count, equal_to(1))

    def test_press_next_source_button(self):
        prev_button = self.get_child_by_label(self.undertest, NavPanel.PREV_LABEL)
        assert_that(prev_button, not_none())

        self.fire_button_click_event(prev_button)
        assert_that(self.controller.on_previous_obs.call_count, equal_to(1))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import wx

from ossos.gui.views.loading import WaitingGaugeDialog


def main():
    class TestFrame(wx.Frame):
        def __init__(self, *args, **kwargs):
            super(TestFrame, self).__init__(*args, **kwargs)

            panel = wx.Panel(self, wx.ID_ANY)

            button = wx.Button(panel, id=wx.ID_ANY, label="Press Me")
            button.Bind(wx.EVT_BUTTON, self.onclick)

        def onclick(self, event):
            self.dlg = WaitingGaugeDialog(self, "Image loading...")
            self.dlg.ShowModal()

    app = wx.App()
    frame = TestFrame(None)
    frame.Show()
    app.MainLoop()


if __name__ == "__main__":
    main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, equal_to

from ossos.fitsviewer.displayable import Marker


class MarkerTest(unittest.TestCase):
    def test_cross_location(self):
        x = 10
        y = 10
        radius = 6

        marker = Marker(x, y, radius)

        assert_that(marker.left_hair.get_xdata(), equal_to((4, 7)))
        assert_that(marker.left_hair.get_ydata(), equal_to((10, 10)))

        assert_that(marker.right_hair.get_xdata(), equal_to((13, 16)))
        assert_that(marker.right_hair.get_ydata(), equal_to((10, 10)))

        assert_that(marker.top_hair.get_xdata(), equal_to((10, 10)))
        assert_that(marker.top_hair.get_ydata(), equal_to((13, 16)))

        assert_that(marker.bottom_hair.get_xdata(), equal_to((10, 10)))
        assert_that(marker.bottom_hair.get_ydata(), equal_to((4, 7)))

    def test_move_marker_moves_circle_and_cross(self):
        x = 10
        y = 10
        radius = 6

        marker = Marker(x, y, radius)

        new_x = 20
        new_y = 30
        marker.center = (new_x, new_y)

        assert_that(marker.circle.center, equal_to((new_x, new_y)))

        assert_that(marker.left_hair.get_xdata(), equal_to((14, 17)))
        assert_that(marker.left_hair.get_ydata(), equal_to((30, 30)))

        assert_that(marker.right_hair.get_xdata(), equal_to((23, 26)))
        assert_that(marker.right_hair.get_ydata(), equal_to((30, 30)))

        assert_that(marker.top_hair.get_xdata(), equal_to((20, 20)))
        assert_that(marker.top_hair.get_ydata(), equal_to((33, 36)))

        assert_that(marker.bottom_hair.get_xdata(), equal_to((20, 20)))
        assert_that(marker.bottom_hair.get_ydata(), equal_to((24, 27)))

    def test_change_radius(self):
        x = 20
        y = 20
        radius = 6

        marker = Marker(x, y, radius)

        new_radius = 12
        marker.radius = new_radius

        assert_that(marker.circle.radius, equal_to(new_radius))

        assert_that(marker.left_hair.get_xdata(), equal_to((8, 14)))
        assert_that(marker.left_hair.get_ydata(), equal_to((20, 20)))

        assert_that(marker.right_hair.get_xdata(), equal_to((26, 32)))
        assert_that(marker.right_hair.get_ydata(), equal_to((20, 20)))

        assert_that(marker.top_hair.get_xdata(), equal_to((20, 20)))
        assert_that(marker.top_hair.get_ydata(), equal_to((26, 32)))

        assert_that(marker.bottom_hair.get_xdata(), equal_to((20, 20)))
        assert_that(marker.bottom_hair.get_ydata(), equal_to((8, 14)))

    def test_toggle_reticule(self):
        marker = Marker(10, 10, 10)

        def assert_visible(visible):
            assert_that(marker.circle.get_visible(), equal_to(visible))
            for line in marker.lines:
                assert_that(line.get_visible(), equal_to(visible))

        assert_visible(True)

        marker.toggle_reticule()

        assert_visible(False)

        marker.toggle_reticule()

        assert_visible(True)


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from mock import Mock, call
from hamcrest import assert_that, equal_to, contains

from ossos import astrom
from ossos.downloads.cutouts.grid import CutoutGrid
from ossos.downloads.cutouts.source import SourceCutout


class CutoutGridPrefilledTest(unittest.TestCase):
    def setUp(self):
        self.source = Mock(spec=astrom.Source)
        self.source.num_readings.return_value = 2

        self.grid = CutoutGrid(self.source)

        self.cutout00 = Mock(spec=SourceCutout)
        self.cutout01 = Mock(spec=SourceCutout)
        self.cutout10 = Mock(spec=SourceCutout)
        self.cutout11 = Mock(spec=SourceCutout)

        self.grid.add_cutout(self.cutout00, 0, 0)
        self.grid.add_cutout(self.cutout01, 0, 1)
        self.grid.add_cutout(self.cutout10, 1, 0)
        self.grid.add_cutout(self.cutout11, 1, 1)

    def test_added_cutouts_in_correct_location(self):
        assert_that(self.grid.get_cutout(0, 0), equal_to(self.cutout00))
        assert_that(self.grid.get_cutout(0, 1), equal_to(self.cutout01))
        assert_that(self.grid.get_cutout(1, 0), equal_to(self.cutout10))
        assert_that(self.grid.get_cutout(1, 1), equal_to(self.cutout11))

    def test_apply_function(self):
        function = Mock()

        self.grid.apply(function)

        assert_that(function.call_args_list, contains(
            call(self.cutout00, 0, 0),
            call(self.cutout01, 0, 1),
            call(self.cutout10, 1, 0),
            call(self.cutout11, 1, 1)))

    def test_reset_source_location(self):
        self.grid.reset_source_location()

        self.cutout00.reset_source_location.called_once_with()
        self.cutout01.reset_source_location.called_once_with()
        self.cutout10.reset_source_location.called_once_with()
        self.cutout11.reset_source_location.called_once_with()


class CutoutGridTest(unittest.TestCase):
    def test_is_filled(self):
        source = Mock(spec=astrom.Source)
        source.num_readings.return_value = 2

        grid = CutoutGrid(source)

        def add_cutout(frame_index, time_index):
            grid.add_cutout(Mock(spec=SourceCutout), frame_index, time_index)

        assert_that(grid.is_filled(), equal_to(False))

        add_cutout(0, 0)
        assert_that(grid.is_filled(), equal_to(False))
        add_cutout(1, 0)
        assert_that(grid.is_filled(), equal_to(False))
        add_cutout(1, 1)
        assert_that(grid.is_filled(), equal_to(False))
        add_cutout(0, 1)
        assert_that(grid.is_filled(), equal_to(True))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from mock import Mock, call
from hamcrest import assert_that, equal_to, has_length, contains

from tests.base_tests import WxWidgetTestCase
from ossos.gui.controllers import ProcessRealsController
from ossos.gui.views.validation import SourceValidationDialog, AcceptSourceDialog, RejectSourceDialog

# Constants used for test data
TEST_MINOR_PLANET_NUMBER = "mpn01"
TEST_PROVISIONAL_NAME = "provisional-name-1"
TEST_DISCOVERY_AST = "*"
TEST_NOTE1 = "A"
TEST_NOTE2 = "B"
TEST_DATE = "2012 01 01"
TEST_DEC = 31.2123
TEST_RA = 27.213
TEST_MAG = "123.5"
TEST_MAG_ERR = "5"
TEST_BAND = "A"
TEST_OBS_CODE = "523"
TEST_COMMENT = "Test comment"


class AcceptSourceDialogTest(WxWidgetTestCase):
    def setUp(self):
        super(AcceptSourceDialogTest, self).setUp()

        self.controller = Mock(spec=ProcessRealsController)

    def create_undertest(self, note1_choices=None, note2_choices=None):
        return AcceptSourceDialog(self.rootframe, self.controller,
                                  TEST_PROVISIONAL_NAME,
                                  TEST_DATE, TEST_RA, TEST_DEC, TEST_MAG,
                                  TEST_MAG_ERR, TEST_BAND,
                                  note1_choices=note1_choices,
                                  note2_choices=note2_choices)

    def test_create_components(self):
        undertest = self.create_undertest()

        component_labels = [AcceptSourceDialog.MINOR_PLANET_NUMBER, AcceptSourceDialog.PROVISIONAL_NAME,
                            AcceptSourceDialog.NOTE2, AcceptSourceDialog.DATE_OF_OBS,
                            AcceptSourceDialog.RA, AcceptSourceDialog.DEC, AcceptSourceDialog.OBS_MAG,
                            AcceptSourceDialog.BAND, AcceptSourceDialog.OBSERVATORY_CODE,
                            AcceptSourceDialog.SUBMIT_BTN, AcceptSourceDialog.CANCEL_BTN]

        for label in component_labels:
            self.assert_has_child_with_label(undertest, label)

    def test_required_preset_values(self):
        undertest = self.create_undertest()

        def assert_has_value(name, value):
            assert_that(self.get_child_by_name(undertest, name).GetValue(),
                        equal_to(str(value)))

        assert_that(undertest.GetTitle(), equal_to(AcceptSourceDialog.TITLE))
        assert_has_value(AcceptSourceDialog.PROVISIONAL_NAME, TEST_PROVISIONAL_NAME)

        assert_has_value(AcceptSourceDialog.RA, TEST_RA)
        assert_has_value(AcceptSourceDialog.DEC, TEST_DEC)
        assert_has_value(AcceptSourceDialog.DATE_OF_OBS, TEST_DATE)

    def test_note_comboboxes_populated(self):
        note1_choices = ["", "n1a", "n1b"]
        note2_choices = ["", "n2a", "n2b", "n2c"]
        undertest = self.create_undertest(note1_choices, note2_choices)

        note1_combobox = self.get_child_by_name(undertest, AcceptSourceDialog.NOTE1)
        assert_that(note1_combobox.GetValue(), equal_to(""))
        assert_that(note1_combobox.GetCount(), equal_to(len(note1_choices)))

        note2_combobox = self.get_child_by_name(undertest, AcceptSourceDialog.NOTE2)
        assert_that(note2_combobox.GetValue(), equal_to(""))
        assert_that(note2_combobox.GetCount(), equal_to(len(note2_choices)))

    def test_cancel_event(self):
        undertest = self.create_undertest()

        cancel_button = self.get_child_by_name(undertest, AcceptSourceDialog.CANCEL_BTN)
        self.fire_button_click_event(cancel_button)

        assert_that(self.controller.on_do_accept.call_args_list, has_length(0))
        assert_that(self.controller.on_cancel_accept.call_args_list, contains(call()))

    def test_submit_data(self):
        note1_choices = ["C", TEST_NOTE1]
        note2_choices = [TEST_NOTE2, "D", "E"]
        undertest = self.create_undertest(note1_choices, note2_choices)

        def get(name):
            """Convenience method"""
            return self.get_child_by_name(undertest, name)

        # Enter data
        get(AcceptSourceDialog.MINOR_PLANET_NUMBER).SetValue(TEST_MINOR_PLANET_NUMBER)
        get(AcceptSourceDialog.NOTE1).SetStringSelection(TEST_NOTE1)
        get(AcceptSourceDialog.NOTE2).SetStringSelection(TEST_NOTE2)
        get(AcceptSourceDialog.OBSERVATORY_CODE).SetValue(TEST_OBS_CODE)
        get(AcceptSourceDialog.COMMENT).SetValue(TEST_COMMENT)

        # Submit data
        ok_button = self.get_child_by_name(
            undertest, SourceValidationDialog.SUBMIT_BTN)
        self.fire_button_click_event(ok_button)

        # Check data
        assert_that(self.controller.on_cancel_accept.called, equal_to(False))
        self.controller.on_do_accept.assert_called_once_with(
            TEST_MINOR_PLANET_NUMBER, TEST_PROVISIONAL_NAME,
            TEST_NOTE1, TEST_NOTE2, TEST_DATE, str(TEST_RA), str(TEST_DEC), TEST_MAG,
            TEST_MAG_ERR, TEST_BAND, TEST_OBS_CODE, TEST_COMMENT)

    def test_submit_data_phot_failure(self):
        undertest = AcceptSourceDialog(self.rootframe, self.controller,
                                       TEST_PROVISIONAL_NAME,
                                       TEST_DATE, TEST_RA, TEST_DEC, TEST_MAG,
                                       TEST_MAG_ERR, TEST_BAND,
                                       note1_choices=None,
                                       note2_choices=None,
                                       default_observatory_code=TEST_OBS_CODE,
                                       default_comment=TEST_COMMENT,
                                       phot_failure=True
        )

        # Submit data
        ok_button = self.get_child_by_name(
            undertest, SourceValidationDialog.SUBMIT_BTN)
        self.fire_button_click_event(ok_button)

        # Check data
        obs_mag = ""
        obs_mag_err = -1
        band = ""
        assert_that(self.controller.on_cancel_accept.called, equal_to(False))
        self.controller.on_do_accept.assert_called_once_with(
            "", TEST_PROVISIONAL_NAME,
            "", "", TEST_DATE, str(TEST_RA), str(TEST_DEC), obs_mag,
            obs_mag_err, band, TEST_OBS_CODE, TEST_COMMENT)


class RejectSourceDialogTest(WxWidgetTestCase):
    def setUp(self):
        super(RejectSourceDialogTest, self).setUp()

        self.controller = Mock(spec=ProcessRealsController)
        self.undertest = RejectSourceDialog(self.rootframe, self.controller)

    def test_submit_comment(self):
        # Enter data
        self.get_child_by_name(
            self.undertest, RejectSourceDialog.COMMENT).SetValue(TEST_COMMENT)

        # Submit data
        ok_button = self.get_child_by_name(
            self.undertest, SourceValidationDialog.SUBMIT_BTN)
        self.fire_button_click_event(ok_button)

        self.controller.on_do_reject.assert_called_once_with(TEST_COMMENT)


if __name__ == '__main__':
    unittest.main()
